package com.dev.lavin.bcard.adapters

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bumptech.glide.Glide
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.utils.TextUtils
import com.dev.lavin.domain2.model.Card
import kotlinx.android.synthetic.main.card_select_item.view.*


/**
 * Created by lavin
 * on 23/04/2018.
 */
class CardListAdapter(private val context: Context, private var cardList: List<Card>) : RecyclerView.Adapter<CardListAdapter.CardListViewHolder>() {

    private var selectedPosition = -1

    companion object {
        const val FIRST_STATE = 0
        const val ELSE_STATE = 1
    }

    override fun getItemViewType(position: Int): Int {
        return if (position == 0) {
            EventsAdapter.FIRST_STATE
        } else {
            EventsAdapter.ELSE_STATE
        }
    }

    //Return the selectedPosition item
    val selectedItem: Card?
        get() {
            if (selectedPosition != -1) {
                //Toast.makeText(context, "Selected Item : " + cardList!![selectedPosition], Toast.LENGTH_SHORT).show()
                return cardList[selectedPosition]
            }
            return null
        }

    inner class CardListViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        fun bind(card: Card, i: Int) {
            //check the radio button if both position and selectedPosition matches
            itemView.radioButton.isChecked = i == selectedPosition
            //Set the position tag to both radio button and label
            itemView.CardContainerRow.tag = i
            itemView.radioButton.isClickable = false
            itemView.CardContainerRow.setOnClickListener { v -> itemCheckChanged(v) }
            if (card.name != null) {
                itemView.CardListName.text = TextUtils.toTitleCase(card.name)
            }
            if (card.position != null) {
                itemView.CardListPosition.text = card.position
            }
            if (card.company != null) {
                itemView.CardListCompany.text = card.companyFormmated
            }
            if (card.userImageUrl != null) {
                Glide.with(context!!).load(card.userImageUrl).into(itemView.CardListUserImage)
            }
        }
    }

    override fun onCreateViewHolder(viewGroup: ViewGroup, i: Int): CardListViewHolder {
        val v = if (i == EventsAdapter.FIRST_STATE) {
            LayoutInflater.from(viewGroup.context).inflate(R.layout.card_select_item_first, viewGroup, false)
        } else {
            LayoutInflater.from(viewGroup.context).inflate(R.layout.card_select_item, viewGroup, false)
        }
        return CardListViewHolder(v)
    }

    override fun getItemCount(): Int {
        return cardList.size
    }

    override fun onBindViewHolder(holder: CardListViewHolder, i: Int) {
        holder.bind(cardList[i], i)
    }

    //On selecting any view set the current position to selectedPositon and notify adapter
    private fun itemCheckChanged(v: View) {
        selectedPosition = v.tag as Int
        notifyDataSetChanged()
    }
}
