package com.dev.lavin.bcard.adapters

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bumptech.glide.Glide
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.utils.TextUtils
import com.dev.lavin.domain2.model.Card
import kotlinx.android.synthetic.main.card_item.view.*

/**
 * Created by lavin
 * on 20/04/2018.
 */
class CardsContactsAdapter(var cardList: List<Card>, private val mListener: CardsContactsAdapter.OnCardSelectedListener) : RecyclerView.Adapter<CardsContactsAdapter.ViewHolder>() {

    interface OnCardSelectedListener {
        fun onCardSelected(card: Card)
    }

    companion object {
        const val FIRST_STATE = 0
        const val ELSE_STATE = 1
    }

    var context: Context? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = if (viewType == EventsAdapter.FIRST_STATE) {
            LayoutInflater.from(parent.context).inflate(R.layout.card_contact_item_first, parent, false)
        } else {
            LayoutInflater.from(parent.context).inflate(R.layout.card_contact_item, parent, false)
        }
        context = parent.context
        return ViewHolder(view)
    }

    override fun getItemViewType(position: Int): Int {
        return if (position == 0) {
            EventsAdapter.FIRST_STATE
        } else {
            EventsAdapter.ELSE_STATE
        }
    }

    override fun onBindViewHolder(holder: CardsContactsAdapter.ViewHolder, position: Int) {
        holder.bind(cardList[position], mListener)
    }

    override fun getItemCount(): Int {
        return cardList.size
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind(card: Card,
                 listener: CardsContactsAdapter.OnCardSelectedListener?) {

            if (card.name != null) {
                itemView.CardName.text = TextUtils.toTitleCase(card.name)
            }
            if (card.position != null) {
                itemView.CardPosition.text = card.position
            }
            if (card.company != null) {
                itemView.CardCompany.text = card.companyFormmated
            }
            if (card.userImageUrl != null) {
                Glide.with(context!!).load(card.userImageUrl).into(itemView.CardUserImage)
            }
            // Click listener
            itemView.setOnClickListener {
                if (listener != null) {
                    listener!!.onCardSelected(card)
                }
            }
        }

    }

}