package com.dev.lavin.bcard.adapters

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bumptech.glide.Glide
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.utils.TextUtils
import com.dev.lavin.domain2.model.Card
import kotlinx.android.synthetic.main.card_event_item.view.*

/**
 * Created by lavin
 * on 20/04/2018.
 */
class CardsEventAdapter(var userId: String, var cardList: List<Card>, private val mListener: CardsEventAdapter.OnCardEventSelectedListener) : RecyclerView.Adapter<CardsEventAdapter.ViewHolder>() {

    interface OnCardEventSelectedListener {
        fun onCardSelected(card: Card)
        fun onAddCardSelected(card: Card)
    }

    var context: Context? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(R.layout.card_event_item, parent, false)
        context = parent.context
        return ViewHolder(view)
    }


    override fun onBindViewHolder(holder: CardsEventAdapter.ViewHolder, position: Int) {
        holder.bind(cardList[position], mListener)
    }

    override fun getItemCount(): Int {
        return cardList.size
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind(card: Card,
                 listener: CardsEventAdapter.OnCardEventSelectedListener?) {

            if (card.userId == userId) {
                itemView.CardEventAddButton.visibility = View.GONE
            }
            if (card.name != null) {
                itemView.CardEventName.text = TextUtils.toTitleCase(card.name)
            }
            if (card.position != null) {
                itemView.CardEventPosition.text = card.position
            }
            if (card.company != null) {
                itemView.CardEventCompany.text = card.companyFormmated
            }
            if (card.userImageUrl != null) {
                Glide.with(context!!).load(card.userImageUrl).into(itemView.CardEventUserImage)
            }
            itemView.CardEventAddButton.setOnClickListener {
                if (listener != null) {
                    listener!!.onAddCardSelected(card)
                }
            }
            // Click listener
            itemView.setOnClickListener {
                if (listener != null) {
                    listener!!.onCardSelected(card)
                }
            }
        }

    }

}