package com.dev.lavin.bcard.adapters

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.text.format.DateFormat
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.dev.lavin.bcard.R
import com.dev.lavin.domain2.model.Event
import com.google.firebase.firestore.FirebaseFirestore
import kotlinx.android.synthetic.main.item_event.view.*
import java.util.*

/**
 * Created by lavin
 * on 20/04/2018.
 */
class EventsAdapter(var eventList: List<Event>, private val mListener: EventsAdapter.OnEventSelectedListener, private val mFireStore: FirebaseFirestore?) : RecyclerView.Adapter<EventsAdapter.ViewHolder>() {

    var context: Context? = null
    var firebaseFirestore: FirebaseFirestore? = null

    companion object {
        const val FIRST_STATE = 0
        const val ELSE_STATE = 1
    }

    interface OnEventSelectedListener {
        fun onEventSelected(event: Event)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = if (viewType == FIRST_STATE) {
            LayoutInflater.from(parent.context).inflate(R.layout.item_event_first, parent, false)
        } else {
            LayoutInflater.from(parent.context).inflate(R.layout.item_event, parent, false)
        }
        context = parent.context
        firebaseFirestore = mFireStore
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(eventList[position], mListener)
    }

    override fun getItemViewType(position: Int): Int {
        return if (position == 0) {
            FIRST_STATE
        } else {
            ELSE_STATE
        }
    }

    override fun getItemCount(): Int {
        return eventList.size
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bind(event: Event,
                 listener: EventsAdapter.OnEventSelectedListener?) {

            // Load image
            val requestOptions = RequestOptions()
            requestOptions.placeholder(R.drawable.event_img_empty_card)
            Glide.with(itemView.EventPicture!!.context)
                    .applyDefaultRequestOptions(requestOptions)
                    .load(event.eventImageUrl)
                    .into(itemView.EventPicture!!)

            itemView.EventTitle!!.text = event.name
            itemView.EventLocation!!.text = event.locationFormatted

            itemView.EventCategory!!.text = event.subject

            if (event.timestamp != null) {
                val millisecond = event.timestamp!!.time
                val dateString = DateFormat.format("dd/MM/yyyy", Date(millisecond)).toString()
                itemView.EventDate!!.text = dateString
            }

            val userId = event.user_id
            if (userId != null && firebaseFirestore != null) {
                firebaseFirestore!!.collection("Users").document(userId).get().addOnCompleteListener({ task ->
                    if (task.isSuccessful) {
                        val userName = task.result.getString("name")
                        val userImage = task.result.getString("image")

                        val requestOptionsUser = RequestOptions()
                        requestOptionsUser.placeholder(R.drawable.card_img_empty_user)
                        itemView.EventOwnerName.text = userName
                        Glide.with(itemView.EventOwnerAvatar!!.context)
                                .applyDefaultRequestOptions(requestOptionsUser)
                                .load(userImage)
                                .into(itemView.EventOwnerAvatar!!)
                    } else {

                        //Firebase Exception

                    }
                })
            }
            // Click listener
            itemView.setOnClickListener {
                if (listener != null) {
                    listener!!.onEventSelected(event)
                }
            }
        }

    }

}