package com.dev.lavin.bcard.blueetooth

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.app.AlertDialog
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.provider.Settings
import android.support.v4.app.ActivityCompat
import android.support.v4.content.ContextCompat
import android.support.v7.app.AppCompatActivity
import android.text.Html
import android.text.method.LinkMovementMethod
import android.util.Log
import android.view.MenuItem
import android.view.View
import android.widget.TextView
import android.widget.Toast
import com.bumptech.glide.Glide
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.utils.TextUtils
import com.dev.lavin.domain2.model.Card
import kotlinx.android.synthetic.main.activity_bluetooth_share.*
import org.json.JSONObject


class BluetoothShareActivity : AppCompatActivity() {

    private var mBluetoothAdapter: BluetoothAdapter? = null
    private var card: Card? = null
    private var mChatService: BluetoothShareService? = null
    private var mOutStringBuffer: StringBuffer? = null
    private var mConnectedDeviceName: String? = null
    private var messageCard: String? = null
    private var MY_PERMISSIONS_REQUEST_ACCESS_COARSE_LOCATION: Int = 1


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_bluetooth_share)
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
        setSupportActionBar(BluetoothShareToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        card = intent?.extras?.getSerializable(BluetoothShareActivity.KEY_CARD) as Card?
        if (card != null) {
            setUpCardInfo(card!!)
            FindDevices?.setOnClickListener {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {  // Only ask for these permissions on runtime when running Android 6.0 or higher
                    when (ContextCompat.checkSelfPermission(baseContext, Manifest.permission.ACCESS_COARSE_LOCATION)) {
                        PackageManager.PERMISSION_DENIED -> (AlertDialog.Builder(this)
                                .setTitle("Permissions Required")
                                .setMessage(Html.fromHtml("<p>To find nearby bluetooth devices please click \"Allow\" on the runtime permissions popup.</p>" + "<p>For more info see <a href=\"http://developer.android.com/about/versions/marshmallow/android-6.0-changes.html#behavior-hardware-id\">here</a>.</p>"))
                                .setNegativeButton("Okay", { _, _ ->
                                    if (ContextCompat.checkSelfPermission(baseContext, Manifest.permission.ACCESS_COARSE_LOCATION) !== PackageManager.PERMISSION_GRANTED) {
                                        ActivityCompat.requestPermissions(this@BluetoothShareActivity,
                                                arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION),
                                                MY_PERMISSIONS_REQUEST_ACCESS_COARSE_LOCATION)
                                    }
                                })
                                .show()
                                .findViewById(android.R.id.message) as TextView).movementMethod = LinkMovementMethod.getInstance()       // Make the link clickable. Needs to be called after show(), in order to generate hyperlinks
                        PackageManager.PERMISSION_GRANTED -> {
                            val intent = Intent(this, DeviceListActivity::class.java)
                            startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE)
                        }
                    }
                } else {
                    val intent = Intent(this, DeviceListActivity::class.java)
                    startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE)
                }
            }
            SendCard?.setOnClickListener {
                val json = JSONObject()
                json.put("bwcardusnm", card!!.name)
                json.put("bwcardid", card!!.id)
                json.put("bwcardusid", card!!.userId)
                messageCard = json.toString()
                sendMessageCard(messageCard!!)
            }
        } else {
            finish()
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int,
                                            permissions: Array<String>, grantResults: IntArray) {
        when (requestCode) {
            MY_PERMISSIONS_REQUEST_ACCESS_COARSE_LOCATION -> {
                if (grantResults.isNotEmpty()
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    // permission was granted :)
                    val intent = Intent(this, DeviceListActivity::class.java)
                    startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE)
                } else {
                    // permission was not granted
                    if (ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.ACCESS_COARSE_LOCATION)) {
                        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(this)
                        alertDialogBuilder.setTitle("Permissions Required")
                                .setMessage("If you do not allow this permission, it is possible that you will no be able to discover the device you need")
                                .setPositiveButton("Allow") { _, _ ->
                                    ActivityCompat.requestPermissions(this@BluetoothShareActivity,
                                            arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION),
                                            MY_PERMISSIONS_REQUEST_ACCESS_COARSE_LOCATION)
                                }
                                .setNegativeButton("Cancel") { _, _ -> }
                                .setCancelable(false)
                                .create()
                                .show()
                    } else {
                        val alertDialogBuilder: AlertDialog.Builder = AlertDialog.Builder(this)
                        alertDialogBuilder.setTitle("Permissions Required")
                                .setMessage("You have forcefully denied some of the required permissions " +
                                        "for this action. Please open settings, go to permissions and allow them.")
                                .setPositiveButton("Settings") { _, _ ->
                                    val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS,
                                            Uri.fromParts("package", packageName, null))
                                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                                    startActivity(intent)
                                }
                                .setNegativeButton("Cancel") { _, _ ->
                                    val intent = Intent(this, DeviceListActivity::class.java)
                                    startActivityForResult(intent, REQUEST_CONNECT_DEVICE_SECURE)
                                }
                                .setCancelable(false)
                                .create()
                                .show()
                    }
                }
            }
        }
    }

    fun setUpCardInfo(card: Card) {
        if (card.name != null) {
            CardName.text = TextUtils.toTitleCase(card.name)
        }
        if (card.position != null) {
            CardPosition.text = card.position
        }
        if (card.company != null) {
            CardCompany.text = card.companyFormmated
        }
        if (card.userImageUrl != null) {
            Glide.with(this).load(card.userImageUrl).into(CardUserImage)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    private fun connectDevice(data: Intent?, secure: Boolean) {
        // Get the device MAC address
        if (data != null) {
            DeviceSelectedLayout?.visibility = View.VISIBLE
            val address = data.extras!!
                    .getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS)
            DeviceMac?.text = address
            // Get the BluetoothDevice object
            val device = mBluetoothAdapter?.getRemoteDevice(address)
            // Attempt to connect to the device
            if (device != null) {
                mChatService?.connect(device, secure)
            }
        }
    }

    private val mHandler = @SuppressLint("HandlerLeak")
    object : Handler() {
        override fun handleMessage(msg: Message) {
            when (msg.what) {
                MESSAGE_STATE_CHANGE -> when (msg.arg1) {
                    BluetoothShareService.STATE_CONNECTED -> {
                        setStatus(getString(R.string.title_connected))
                        SendCard?.isEnabled = true
                        //mConversationArrayAdapter.clear()
                    }
                    BluetoothShareService.STATE_CONNECTING -> {
                        setStatus(getString(R.string.title_connecting))
                        SendCard?.isEnabled = false
                    }
                    BluetoothShareService.STATE_LISTEN, BluetoothShareService.STATE_NONE -> {
                        setStatus(getString(R.string.title_not_connected))
                        SendCard?.isEnabled = false
                    }
                }
                MESSAGE_WRITE -> {
                    val writeBuf = msg.obj as ByteArray
                    // construct a string from the buffer
                    val writeMessage = String(writeBuf)
                    //Toast.makeText(this@BluetoothShareActivity, writeMessage, Toast.LENGTH_SHORT).show()
                }
                MESSAGE_READ -> {
                    val readBuf = msg.obj as ByteArray
                    // construct a string from the valid bytes in the buffer
                    val readMessage = String(readBuf, 0, msg.arg1)
                    Toast.makeText(this@BluetoothShareActivity, readMessage, Toast.LENGTH_SHORT).show()
                }
                MESSAGE_DEVICE_NAME -> {
                    // save the connected device's name
                    mConnectedDeviceName = msg.data.getString(Constants.DEVICE_NAME)
                    DeviceName?.text = mConnectedDeviceName
                }
                MESSAGE_TOAST -> {
                    Toast.makeText(this@BluetoothShareActivity, msg.data.getString(Constants.TOAST), Toast.LENGTH_SHORT).show()
                }
            }
        }
    }

    private fun sendMessageCard(message: String) {
        // Check that we're actually connected before trying anything
        if (mChatService?.state !== BluetoothShareService.STATE_CONNECTED) {
            Toast.makeText(this, R.string.title_not_connected, Toast.LENGTH_SHORT).show()
            return
        }
        // Check that there's actually something to send
        if (message.isNotEmpty()) {
            // Get the message bytes and tell the BluetoothChatService to write
            val send = message.toByteArray()
            mChatService?.write(send)
            // Reset out string buffer to zero and clear the edit text field
            mOutStringBuffer?.setLength(0)
        }
    }

    public override fun onStart() {
        super.onStart()
        if (mChatService == null) {
            setupChat()
        }
    }

    public override fun onDestroy() {
        super.onDestroy()
        if (mChatService != null) {
            mChatService!!.stop()
        }
    }

    public override fun onResume() {
        super.onResume()

        // Performing this check in onResume() covers the case in which BT was
        // not enabled during onStart(), so we were paused to enable it...
        // onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
        if (mChatService != null) {
            // Only if the state is STATE_NONE, do we know that we haven't started already
            if (mChatService!!.state === BluetoothShareService.STATE_NONE) {
                // Start the Bluetooth chat services
                mChatService!!.start()
            }
        }
    }

    private fun setStatus(subTitle: String) {
        DeviceStatus?.text = subTitle
    }

    private fun setupChat() {
        Log.d("test", "setupChat()")
        // Initialize the BluetoothChatService to perform bluetooth connections
        mChatService = BluetoothShareService(this, mHandler)

        // Initialize the buffer for outgoing messages
        mOutStringBuffer = StringBuffer("")
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == BluetoothShareActivity.REQUEST_CONNECT_DEVICE_SECURE) {
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data, true)
            }
        }
    }//onActivityResult

    companion object {
        const val KEY_CARD = "key_card"
        const val REQUEST_CONNECT_DEVICE_SECURE = 100
        // Message types sent from the BluetoothChatService Handler
        const val MESSAGE_STATE_CHANGE = 1
        const val MESSAGE_READ = 2
        const val MESSAGE_WRITE = 3
        const val MESSAGE_DEVICE_NAME = 4
        const val MESSAGE_TOAST = 5
        // Key names received from the BluetoothChatService Handler
        const val DEVICE_NAME = "device_name"
        const val TOAST = "toast"
    }
}
