package com.dev.lavin.bcard.di.activity


import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.base.BaseView
import com.dev.lavin.bcard.di.presenter.BasePresenterModule
import com.dev.lavin.bcard.di.scopes.PerActivity
import com.dev.lavin.bcard.login.LoginActivity
import com.dev.lavin.bcard.login.LoginPresenter
import com.dev.lavin.bcard.login.LoginPresenterImpl
import com.dev.lavin.bcard.login.LoginView
import com.dev.lavin.bcard.main.MainActivity
import com.dev.lavin.bcard.main.MainActivityPresenter
import com.dev.lavin.bcard.main.MainActivityPresenterImpl
import com.dev.lavin.bcard.main.MainActivityView
import com.dev.lavin.bcard.main.cards.create.CardCreatePresenter
import com.dev.lavin.bcard.main.cards.create.CardCreatePresenterImpl
import com.dev.lavin.bcard.main.cards.create.CardCreateView
import com.dev.lavin.bcard.main.cards.detail.CardDetailActivity
import com.dev.lavin.bcard.main.cards.detail.CardDetailPresenter
import com.dev.lavin.bcard.main.cards.detail.CardDetailPresenterImpl
import com.dev.lavin.bcard.main.cards.detail.CardDetailView
import com.dev.lavin.bcard.main.cards.edit.EditCardPresenter
import com.dev.lavin.bcard.main.cards.edit.EditCardPresenterImpl
import com.dev.lavin.bcard.main.cards.edit.EditCardView
import com.dev.lavin.bcard.main.events.cardadd.EventCardAddPresenter
import com.dev.lavin.bcard.main.events.cardadd.EventCardAddPresenterImpl
import com.dev.lavin.bcard.main.events.cardadd.EventCardAddView
import com.dev.lavin.bcard.main.events.create.NewEventPresenter
import com.dev.lavin.bcard.main.events.create.NewEventPresenterImpl
import com.dev.lavin.bcard.main.events.create.NewEventView
import com.dev.lavin.bcard.main.events.detail.EventDetailActivity
import com.dev.lavin.bcard.main.events.detail.EventDetailPresenter
import com.dev.lavin.bcard.main.events.detail.EventDetailView
import com.dev.lavin.bcard.main.events.detail.EventsDetailPresenterImpl
import com.dev.lavin.bcard.main.events.edit.EditEventActivity
import com.dev.lavin.bcard.main.events.edit.EditEventPresenter
import com.dev.lavin.bcard.main.events.edit.EditEventPresenterImpl
import com.dev.lavin.bcard.main.events.edit.EditEventView
import com.dev.lavin.bcard.register.RegisterActivity
import com.dev.lavin.bcard.register.RegisterPresenter
import com.dev.lavin.bcard.register.RegisterPresenterImpl
import com.dev.lavin.bcard.register.RegisterView
import com.dev.lavin.bcard.resetpassword.ResetPasswordActivity
import com.dev.lavin.bcard.resetpassword.ResetPasswordPresenter
import com.dev.lavin.bcard.resetpassword.ResetPasswordPresenterImpl
import com.dev.lavin.bcard.resetpassword.ResetPasswordView

import dagger.Module
import dagger.Provides


@Module
class ActivityModule(var baseActivity: BaseActivity<*>, var view: BaseView) {

    @Provides
    @PerActivity
    internal fun provideLoginActivity(): LoginActivity {
        return this.baseActivity as LoginActivity
    }

    @Provides
    @PerActivity
    internal fun provideMainActivity(): MainActivity {
        return this.baseActivity as MainActivity
    }

    @Provides
    @PerActivity
    internal fun provideRegisterActivity(): RegisterActivity {
        return this.baseActivity as RegisterActivity
    }

    @Provides
    @PerActivity
    internal fun provideResetPasswordActivity(): ResetPasswordActivity {
        return this.baseActivity as ResetPasswordActivity
    }

    @Provides
    @PerActivity
    internal fun provideEventDetailActivity(): EventDetailActivity {
        return this.baseActivity as EventDetailActivity
    }

    @Provides
    @PerActivity
    internal fun provideCardDetailActivity(): CardDetailActivity {
        return this.baseActivity as CardDetailActivity
    }

    @Provides
    @PerActivity
    internal fun provideEditEventActivity(): EditEventActivity {
        return this.baseActivity as EditEventActivity
    }

    @Provides
    @PerActivity
    internal fun provideMainActivityPresenter(activityComponent: ActivityComponent): MainActivityPresenter {
        return MainActivityPresenterImpl(view as MainActivityView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideLoginPresenter(activityComponent: ActivityComponent): LoginPresenter {
        return LoginPresenterImpl(view as LoginView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideRegisterPresenter(activityComponent: ActivityComponent): RegisterPresenter {
        return RegisterPresenterImpl(view as RegisterView, activityComponent.with(BasePresenterModule(baseActivity)))
    }


    @Provides
    @PerActivity
    internal fun provideEventDetailPresenter(activityComponent: ActivityComponent): EventDetailPresenter {
        return EventsDetailPresenterImpl(view as EventDetailView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideCardDetailPresenter(activityComponent: ActivityComponent): CardDetailPresenter {
        return CardDetailPresenterImpl(view as CardDetailView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideNewEventPresenter(activityComponent: ActivityComponent): NewEventPresenter {
        return NewEventPresenterImpl(view as NewEventView, activityComponent.with(BasePresenterModule(baseActivity)))
    }


    @Provides
    @PerActivity
    internal fun provideEventCardAddPresenter(activityComponent: ActivityComponent): EventCardAddPresenter {
        return EventCardAddPresenterImpl(view as EventCardAddView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideCardCreatePresenter(activityComponent: ActivityComponent): CardCreatePresenter {
        return CardCreatePresenterImpl(view as CardCreateView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideResetPasswordPresenter(activityComponent: ActivityComponent): ResetPasswordPresenter {
        return ResetPasswordPresenterImpl(view as ResetPasswordView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideEditEventPresenter(activityComponent: ActivityComponent): EditEventPresenter {
        return EditEventPresenterImpl(view as EditEventView, activityComponent.with(BasePresenterModule(baseActivity)))
    }

    @Provides
    @PerActivity
    internal fun provideEditCardPresenter(activityComponent: ActivityComponent): EditCardPresenter {
        return EditCardPresenterImpl(view as EditCardView, activityComponent.with(BasePresenterModule(baseActivity)))
    }
}
