package com.dev.lavin.bcard.di.presenter

import android.app.Activity
import com.dev.lavin.bcard.di.scopes.PerPresenter
import com.dev.lavin.data.di.BaseRepositoryModule
import com.dev.lavin.data.services.*
import com.dev.lavin.domain2.boundaries.*
import com.dev.lavin.domain2.di.BaseInteractorModule
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import com.dev.lavin.domain2.interactors.cards.CardsInteractorImpl
import com.dev.lavin.domain2.interactors.events.EventsInteractor
import com.dev.lavin.domain2.interactors.events.EventsInteractorImpl
import com.dev.lavin.domain2.interactors.login.LoginInteractor
import com.dev.lavin.domain2.interactors.login.LoginInteractorImpl
import com.dev.lavin.domain2.interactors.profile.ProfileInteractor
import com.dev.lavin.domain2.interactors.profile.ProfileInteractorImpl
import com.dev.lavin.domain2.interactors.register.RegisterInteractor
import com.dev.lavin.domain2.interactors.register.RegisterInteractorImpl
import com.dev.lavin.domain2.interactors.resetpassword.ResetPasswordInteractor
import com.dev.lavin.domain2.interactors.resetpassword.ResetPasswordInteractorImpl
import dagger.Module
import dagger.Provides

@Module
class BasePresenterModule(private val activty: Activity) {

    @PerPresenter
    @Provides
    fun provideLoginInteractor(basePresenterComponent: BasePresenterComponent): LoginInteractor {
        return LoginInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }

    @PerPresenter
    @Provides
    fun provideLoginRepoInterface(basePresenterComponent: BasePresenterComponent): LoginRepoInterface {
        return LoginService(basePresenterComponent.with(BaseRepositoryModule()))
    }

    @PerPresenter
    @Provides
    fun provideRegisterInteractor(basePresenterComponent: BasePresenterComponent): RegisterInteractor {
        return RegisterInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }

    @PerPresenter
    @Provides
    fun provideRegisterRepoInterface(basePresenterComponent: BasePresenterComponent): RegisterRepoInterface {
        return RegisterService(basePresenterComponent.with(BaseRepositoryModule()))
    }

    @PerPresenter
    @Provides
    fun provideProfileInteractor(basePresenterComponent: BasePresenterComponent): ProfileInteractor {
        return ProfileInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }

    @PerPresenter
    @Provides
    fun provideProfileRepoInterface(basePresenterComponent: BasePresenterComponent): ProfileRepoInterface {
        return ProfileService(basePresenterComponent.with(BaseRepositoryModule()))
    }

    @PerPresenter
    @Provides
    fun provideCardsInteractor(basePresenterComponent: BasePresenterComponent): CardsInteractor {
        return CardsInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }

    @PerPresenter
    @Provides
    fun provideCardsRepoInterface(basePresenterComponent: BasePresenterComponent): CardsRepoInterface {
        return CardsService(basePresenterComponent.with(BaseRepositoryModule()), activty)
    }

    @PerPresenter
    @Provides
    fun provideEventsInteractor(basePresenterComponent: BasePresenterComponent): EventsInteractor {
        return EventsInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }

    @PerPresenter
    @Provides
    fun provideEventsRepoInterface(basePresenterComponent: BasePresenterComponent): EventsRepoInterface {
        return EventsService(basePresenterComponent.with(BaseRepositoryModule()), activty)
    }

    @PerPresenter
    @Provides
    fun provideResetPasswordRepoInterface(basePresenterComponent: BasePresenterComponent): ResetPasswordRepoInterface {
        return ResetPasswordService(basePresenterComponent.with(BaseRepositoryModule()))
    }

    @PerPresenter
    @Provides
    fun provideResetPasswordInteractor(basePresenterComponent: BasePresenterComponent): ResetPasswordInteractor {
        return ResetPasswordInteractorImpl(basePresenterComponent.with(BaseInteractorModule()))
    }
}
