package com.dev.lavin.bcard.login

import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.main.MainActivity
import com.dev.lavin.bcard.register.RegisterActivity
import com.dev.lavin.bcard.resetpassword.ResetPasswordActivity
import com.google.firebase.auth.FirebaseAuth
import kotlinx.android.synthetic.main.activity_login.*
import javax.inject.Inject


class LoginActivity : BaseActivity<ActivityComponent>(), LoginView {

    @Inject
    lateinit var mAuth: FirebaseAuth
    @Inject
    lateinit var presenter:LoginPresenter

    private var progressDialog: ProgressDialog? = null

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_login)
        setLoginAction()
        setRegisterAction()
        setForgotPassAction()
    }

    override fun onStart() {
        super.onStart()
        val user = mAuth.currentUser
        if (user != null) {
            //  No user is signed in
            sendToMain()
        }
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@LoginActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Logging in")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun showServiceError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_LONG).show()
    }

    override fun showGenericError() {
        Toast.makeText(this, "An unexpected error occurred", Toast.LENGTH_LONG).show()
    }

    override fun showInvalidFieldsError() {
        Toast.makeText(this, "Fields must not be empty", Toast.LENGTH_LONG).show()
    }

    private fun setLoginAction() {
        mLoginBtnLg?.setOnClickListener {
            val loginEmail = mEmailEdtLg?.text.toString()
            val loginPass = mPassEdtLg?.text.toString()
            presenter.tryLogin(loginEmail, loginPass)
        }
    }

    private fun setRegisterAction() {
        mRegisterBtnLg.setOnClickListener { sendToRegister() }
    }

    private fun setForgotPassAction(){
        mForgotPassTvLg.setOnClickListener { sendToResetPassword() }
    }

    override fun sendToMain() {
        val intent = Intent(this, MainActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun sendToRegister() {
        val intent = Intent(this, RegisterActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun sendToResetPassword() {
        val intent = Intent(this, ResetPasswordActivity::class.java)
        startActivity(intent)
        finish()
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus?.windowToken, 0)
    }
}
