package com.dev.lavin.bcard.main

import android.annotation.SuppressLint
import android.app.Activity
import android.app.ProgressDialog
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.support.design.widget.Snackbar
import android.support.v4.app.Fragment
import android.support.v7.app.AlertDialog
import android.util.Log
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.blueetooth.BluetoothShareService
import com.dev.lavin.bcard.blueetooth.Constants
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.login.LoginActivity
import com.dev.lavin.bcard.main.contacts.ContactsFragment
import com.dev.lavin.bcard.main.events.EventsFragment
import com.dev.lavin.bcard.main.events.detail.EventDetailActivity
import com.dev.lavin.bcard.main.profile.ProfileFragment
import com.dev.lavin.bcard.utils.InternetUtils
import com.dev.lavin.domain2.model.Contacts
import com.google.firebase.auth.FirebaseAuth
import kotlinx.android.synthetic.main.activity_main.*
import org.json.JSONObject
import javax.inject.Inject


class MainActivity : BaseActivity<ActivityComponent>(), MainActivityView {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var presenter: MainActivityPresenter

    @Inject
    lateinit var contacts: Contacts

    private var mHomeFragment: ContactsFragment? = null
    private var mEventsFragment: EventsFragment? = null
    private var mProfileFragment: ProfileFragment? = null
    private var mChatService: BluetoothShareService? = null
    private var mOutStringBuffer: StringBuffer? = null
    private var mBluetoothAdapter: BluetoothAdapter? = null
    private var mConnectedDeviceName: String? = null
    private var progressDialog: ProgressDialog? = null


    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onNewIntent(intent: Intent) {
        super.onNewIntent(intent)
        setIntent(intent)
        val user = mAuth?.currentUser
        if (user != null) {
            //Toast.makeText(this@MainActivity, "DeepLink detected", Toast.LENGTH_SHORT).show()
            if (intent.extras == null) {
                return
            } else {
                val bundle = intent.extras
                if (bundle!!.containsKey("EventId")) {
                    val id = bundle.getString("EventId")
                    sendToEventDetail(id)
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setSupportActionBar(MainToolbar)
        var bundle: Bundle? = null
        if (intent.extras != null) {
            //Toast.makeText(this@MainActivity, "DeepLink detected", Toast.LENGTH_SHORT).show()
            bundle = intent.extras
        }
        // FRAGMENTS
        mHomeFragment = ContactsFragment.newInstance()
        mEventsFragment = EventsFragment.newInstance()
        mProfileFragment = ProfileFragment.newInstance()

        initializeFragment()
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()

        MainBottomNav.setOnNavigationItemSelectedListener({ item ->
            val currentFragment = supportFragmentManager.findFragmentById(R.id.main_container)

            when (item.itemId) {

                R.id.bottom_action_home -> {
                    setSupportActionBarTitle(getString(R.string.bottom_cards_text))
                    replaceFragment(mHomeFragment, currentFragment)
                    true
                }

                R.id.bottom_action_events -> {
                    setSupportActionBarTitle(getString(R.string.bottom_events_text))
                    replaceFragment(mEventsFragment, currentFragment)
                    true
                }

                R.id.bottom_action_profile -> {
                    setSupportActionBarTitle(getString(R.string.bottom_profile_text))
                    replaceFragment(mProfileFragment, currentFragment)
                    true
                }
                else -> false
            }
        })
        MainBottomNav.selectedItemId = R.id.bottom_action_home
        if (bundle != null && bundle.containsKey("EventId")) {
            val id = bundle.getString("EventId")
            sendToEventDetail(id)
        }
    }

    private fun setSupportActionBarTitle(title: String) {
        supportActionBar?.title = title
    }

    public override fun onStart() {
        super.onStart()
        val user = mAuth?.currentUser
        if (user == null) {
            //  No user is signed in
            if (mChatService != null) {
                mChatService!!.stop()
            }
            sendToLogin()
        } else {
            // If BT is not on, request that it be enabled.
            // setupChat() will then be called during onActivityResult
            if (mBluetoothAdapter != null && !mBluetoothAdapter!!.isEnabled) {
                val enableIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
                startActivityForResult(enableIntent, MainActivity.REQUEST_ENABLE_BT)
                // Otherwise, setup the chat session
            } else if (mChatService == null) {
                setupChat()
            }
        }
    }

    public override fun onDestroy() {
        super.onDestroy()
        if (mChatService != null) {
            mChatService!!.stop()
        }
    }

    public override fun onResume() {
        super.onResume()
        // Performing this check in onResume() covers the case in which BT was
        // not enabled during onStart(), so we were paused to enable it...
        // onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
        if (mBluetoothAdapter != null && mBluetoothAdapter!!.isEnabled && mChatService == null) {
            setupChat()
        }
        if (mChatService != null) {
            // Only if the state is STATE_NONE, do we know that we haven't started already
            if (mChatService!!.state === BluetoothShareService.STATE_NONE) {
                // Start the Bluetooth chat services
                mChatService!!.start()
            }
        }
    }

    private fun setupChat() {
        Log.d("test", "setupChat()")
        // Initialize the BluetoothChatService to perform bluetooth connections
        mChatService = BluetoothShareService(this, mHandler)

        // Initialize the buffer for outgoing messages
        mOutStringBuffer = StringBuffer("")
    }

    private val mHandler = @SuppressLint("HandlerLeak")
    object : Handler() {
        override fun handleMessage(msg: Message) {
            when (msg.what) {
                MESSAGE_STATE_CHANGE -> when (msg.arg1) {
                    BluetoothShareService.STATE_CONNECTED -> {
                        Toast.makeText(this@MainActivity, getString(R.string.title_connected_to, mConnectedDeviceName), Toast.LENGTH_SHORT).show()
                    }
                    BluetoothShareService.STATE_CONNECTING -> {
                        //Toast.makeText(this@MainActivity, getString(R.string.title_connecting), Toast.LENGTH_SHORT).show()
                    }
                    BluetoothShareService.STATE_LISTEN, BluetoothShareService.STATE_NONE -> {
                        //Toast.makeText(this@MainActivity, getString(R.string.title_not_connected), Toast.LENGTH_SHORT).show()
                    }
                }
                MESSAGE_READ -> {
                    val readBuf = msg.obj as ByteArray
                    // construct a string from the valid bytes in the buffer
                    val readMessage = String(readBuf, 0, msg.arg1)
                    val data = JSONObject(readMessage)
                    if (data.has("bwcardusnm") && data.has("bwcardid") && data.has("bwcardusid")) {
                        Snackbar.make(findViewById(android.R.id.content), "Card received from: " + data.get("bwcardusnm"), Snackbar.LENGTH_LONG).setAction("Accept") {
                            addToContacts(data)
                        }.show()
                    } else {
                        Toast.makeText(this@MainActivity, "You have received invalid data", Toast.LENGTH_SHORT).show()
                    }
                }
                MESSAGE_DEVICE_NAME -> {
                    mConnectedDeviceName = msg.data.getString(Constants.DEVICE_NAME)
                    Toast.makeText(this@MainActivity, "Connected to $mConnectedDeviceName", Toast.LENGTH_SHORT).show()
                }
                MESSAGE_TOAST -> {
                    Toast.makeText(this@MainActivity, msg.data.getString(Constants.TOAST), Toast.LENGTH_SHORT).show()
                }
            }
        }
    }

    fun logOut() {
        contacts.emptyContacts()
        mAuth.signOut()
        sendToLogin()
    }

    private fun sendToLogin() {
        val intent = Intent(this, LoginActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun initializeFragment() {
        val fragmentTransaction = supportFragmentManager.beginTransaction()
        fragmentTransaction.add(R.id.main_container, mHomeFragment)
        fragmentTransaction.add(R.id.main_container, mEventsFragment)
        fragmentTransaction.add(R.id.main_container, mProfileFragment)
        fragmentTransaction.commit()
    }

    private fun replaceFragment(fragment: Fragment?, currentFragment: Fragment?) {

        val fragmentTransaction = supportFragmentManager.beginTransaction()
        if (fragment === mHomeFragment) {
            fragmentTransaction.hide(mProfileFragment)
            fragmentTransaction.hide(mEventsFragment)
        }
        if (fragment === mProfileFragment) {
            fragmentTransaction.hide(mHomeFragment)
            fragmentTransaction.hide(mEventsFragment)
        }
        if (fragment === mEventsFragment) {
            fragmentTransaction.hide(mHomeFragment)
            fragmentTransaction.hide(mProfileFragment)
        }
        fragmentTransaction.show(fragment)
        fragmentTransaction.commit()
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == MainActivity.REQUEST_ENABLE_BT) {
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                setupChat()
            } else {
                // User did not enable Bluetooth or an error occurred
                Toast.makeText(this, "You must enable bluetooth to share and receive cards",
                        Toast.LENGTH_SHORT).show()
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data)
        }
    }

    override fun setConnectionError() {
        Toast.makeText(this, "Check your connectivity", Toast.LENGTH_LONG).show()
    }

    override fun showError(message: String) {
        Snackbar.make(findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    private fun addToContacts(data: JSONObject) {
        if (data.has("bwcardid") && data.has("bwcardusid")) {
            if (data.getString("bwcardusid") != mAuth.currentUser!!.uid) {
                if (InternetUtils.checkInternetConnection(this)) {
                    presenter.addCardToContacts(data.getString("bwcardid"), mAuth.currentUser!!.uid)
                } else {
                    setConnectionError()
                }
            } else {
                Toast.makeText(this, "You are trying yo add your own card", Toast.LENGTH_LONG).show()
            }
        } else {
            Toast.makeText(this, "Incorrect data", Toast.LENGTH_LONG).show()
        }
    }

    override fun showAddLoading() {
        progressDialog = ProgressDialog(this)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Adding the card to your contacts")
        progressDialog?.show()
    }

    override fun hideAddLoading() {
        progressDialog?.dismiss()
    }

    override fun onCardAdded() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("The card has been added")
                .setPositiveButton("Accept", { _, _ ->

                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    fun sendToEventDetail(id: String) {
        val intent = Intent(this, EventDetailActivity::class.java)
        intent.putExtra(EventDetailActivity.KEY_EVENT_ID, id)
        startActivity(intent)
    }

    companion object {
        const val REQUEST_ENABLE_BT = 150
        // Message types sent from the BluetoothChatService Handler
        const val MESSAGE_STATE_CHANGE = 1
        const val MESSAGE_READ = 2
        const val MESSAGE_DEVICE_NAME = 4
        const val MESSAGE_TOAST = 5
    }
}
