package com.dev.lavin.bcard.main.cards.create

import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v7.app.AlertDialog
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.utils.FloatingButton
import com.dev.lavin.bcard.utils.ImageUtils
import kotlinx.android.synthetic.main.activity_card_create.*
import java.io.FileNotFoundException
import javax.inject.Inject


/**
 * Created by lavin
 * on 25/04/2018.
 */
class CardCreateActivity : BaseActivity<ActivityComponent>(), CardCreateView, FloatingButton.IEvents {

    private var imageProfileUri: Uri? = null

    private var IMAGE_PROFILE_STATUS = false

    private var imageCaptionUri: Uri? = null

    private var IMAGE_CAPTION_STATUS = false

    private var progressDialog: ProgressDialog? = null

    @Inject
    lateinit var presenter: CardCreatePresenter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_card_create)
        setUp()
    }

    override fun setUp() {
        setSupportActionBar(CardCreateToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        mSaveBtnCc?.setupScrollListeners(mAppBarCc, mNestedCc)
        mSaveBtnCc?.setButtonHideEnable(true)
        mSaveBtnCc?.setButtonEnabled(true)
        mSaveBtnCc?.setEvents(this)
        mImageCc.setOnClickListener { BringProfileImagePicker() }
        mCaptionCc.setOnClickListener { BringCaptionImagePicker() }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    private fun BringProfileImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 1000)
    }

    private fun BringCaptionImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 2000)
    }

    override fun setErrorService(message: String) {
        Snackbar.make(findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@CardCreateActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Creating your card")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }

    override fun showInvalidFieldsError() {
        Snackbar.make(this.findViewById(android.R.id.content), "Fields must not be empty", Snackbar.LENGTH_SHORT).show()
    }

    override fun onCardCreated() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your card has been created")
                .setPositiveButton("Accept", { _, _ ->
                    finish()
                    overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    override fun showInvalidEmail() {
        Toast.makeText(this, "Enter a valid email", Toast.LENGTH_LONG).show()
    }

    override fun showInvalidWeb() {
        Toast.makeText(this, "Enter a valid website", Toast.LENGTH_LONG).show()
    }

    override fun onButtonClick() {
        val newCardName = mNameEdtCc?.text.toString()
        val newCardCompany = mCompanyEdtCc?.text.toString()
        val newCardPosition = mPositionEdtCc?.text.toString()
        val newCardEmail = mEmailEdtCc?.text.toString()
        val newCardPhone = mPhoneEdtCc?.text.toString()
        val newCardWeb = mWebEdtCc?.text.toString()
        val newCardNotes = mNotesEdtCc?.text.toString()
        presenter.createCard(newCardName, newCardCompany, newCardPosition, newCardEmail, newCardPhone, newCardWeb, newCardNotes, imageProfileUri, imageCaptionUri)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageProfileUri = data.data//Getting uri of the data
                val resultProfile = ImageUtils.decodeUri(imageProfileUri!!, this)
                mImageCc?.setImageBitmap(resultProfile)
                IMAGE_PROFILE_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }
        }
        if (requestCode == 2000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageCaptionUri = data.data//Getting uri of the data
                val resultCaption = ImageUtils.decodeUri(imageCaptionUri!!, this)
                mCaptionCc?.setImageBitmap(resultCaption)
                IMAGE_CAPTION_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }
        }
    }
}