package com.dev.lavin.bcard.main.cards.create

import android.net.Uri
import android.text.TextUtils
import android.util.Patterns
import android.webkit.URLUtil
import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import java.net.MalformedURLException
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class CardCreatePresenterImpl(cardCreateView: CardCreateView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<CardCreateView>(cardCreateView), CardCreatePresenter, CardsInteractor.CreateCardsInteractorResponse {

    @Inject
    lateinit var cardsInteractor: CardsInteractor

    init {
        view = cardCreateView
        basePresenterComponent.injectPresenter(this)
    }

    override fun createCard(name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?) {
        if (!TextUtils.isEmpty(name) && !TextUtils.isEmpty(company) && !TextUtils.isEmpty(position) && !TextUtils.isEmpty(email) && !TextUtils.isEmpty(phone) && profile != null) {
            if (isEmailValid(email)) {
                if (!web.isEmpty() && isValidWeb(web)) {
                    view?.hideKeyboard()
                    view?.showLoading()
                    cardsInteractor.createCard(name, company, position, email, phone, web, notes, profile, caption, this)
                } else if (web.isEmpty()) {
                    view?.hideKeyboard()
                    view?.showLoading()
                    cardsInteractor.createCard(name, company, position, email, phone, web, notes, profile, caption, this)
                } else {
                    view?.showInvalidWeb()
                }
            } else {
                view?.showInvalidEmail()
            }
        } else {
            view?.showInvalidFieldsError()
        }
    }

    override fun onCreateCardsInteractorResponseSuccess() {
        view?.hideLoading()
        view.onCardCreated()
    }

    override fun onCreateCardsInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.setErrorService(message)
    }

    fun isValidWeb(urlString: String): Boolean {
        return try {
            URLUtil.isValidUrl(urlString) && Patterns.WEB_URL.matcher(urlString).matches()
        } catch (e: MalformedURLException) {
            e.printStackTrace()
            false
        }
    }

    private fun isEmailValid(email: String): Boolean {
        return email.contains("@")
    }

}