package com.dev.lavin.bcard.main.cards.detail

import android.app.Activity
import android.app.ProgressDialog
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.BottomSheetDialog
import android.support.design.widget.Snackbar
import android.support.v7.app.AlertDialog
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.Toast
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.blueetooth.BluetoothShareActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.main.cards.edit.EditCardActivity
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.Contacts
import com.google.firebase.auth.FirebaseAuth
import com.google.zxing.BarcodeFormat
import com.google.zxing.MultiFormatWriter
import com.google.zxing.WriterException
import com.google.zxing.common.BitMatrix
import com.journeyapps.barcodescanner.BarcodeEncoder
import kotlinx.android.synthetic.main.activity_card_detail.*
import kotlinx.android.synthetic.main.card_detail_data_row.*
import kotlinx.android.synthetic.main.card_detail_profile_info.*
import org.json.JSONObject
import javax.inject.Inject


class CardDetailActivity : BaseActivity<ActivityComponent>(), CardDetailView {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var presenter: CardDetailPresenter

    @Inject
    lateinit var contacts: Contacts

    var card: Card? = null
    var isEvent: Boolean? = false
    private var progressDialog: ProgressDialog? = null
    private var mBluetoothAdapter: BluetoothAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_card_detail)
        setSupportActionBar(CardDetailToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        isEvent = intent?.extras?.getBoolean(CardDetailActivity.KEY_CARD_FROM)
        card = intent?.extras?.getSerializable(CardDetailActivity.KEY_CARD) as Card?
        if (card != null) {
            // Get local Bluetooth adapter
            mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
            presenter.presentInfo(card!!)
        } else {
            finish()
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.card_detail_menu, menu)
        if (isEvent!!) {
            if (mAuth.currentUser != null && card!!.userId != mAuth.currentUser!!.uid) {
                val addItem = menu.findItem(R.id.action_card_add_btn)
                addItem?.isVisible = true
            } else {
                val shareItem = menu.findItem(R.id.action_card_share_btn)
                shareItem?.isVisible = true
            }
        }
        if (card != null && mAuth.currentUser != null && card!!.userId == mAuth.currentUser!!.uid && !(isEvent!!)) {
            val item = menu.findItem(R.id.action_card_edit_btn)
            item?.isVisible = true
            val shareItem = menu.findItem(R.id.action_card_share_btn)
            shareItem?.isVisible = true
        }
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            R.id.action_card_add_btn -> {
                if (card != null && mAuth.currentUser != null) {
                    if (contacts != null && !contacts.checkIfAlreadyContacts(card!!.id!!)) {
                        presenter.addCardToContacts(card!!.id!!, mAuth.currentUser!!.uid)
                    } else {
                        Toast.makeText(this, "The card is already added in your contacts", Toast.LENGTH_LONG).show()
                    }
                }
                true
            }
            R.id.action_card_edit_btn -> {
                if (card != null) {
                    sendToEdit(card!!)
                }
                true
            }
            R.id.action_card_share_btn -> {
                val contentView = layoutInflater.inflate(R.layout.fragment_bottom_sheet_dialog, null)
                val dialog = BottomSheetDialog(this)
                dialog.setContentView(contentView)
                dialog.show()
                val qr = contentView.findViewById(R.id.bottom_sheet_qr) as LinearLayout
                val bluetooth = contentView.findViewById(R.id.bottom_sheet_bluetooth) as LinearLayout
                qr.setOnClickListener {
                    dialog.dismiss()
                    val contentViewQR = layoutInflater.inflate(R.layout.fragment_bottom_sheet_qr_dialog, null)
                    val dialogQR = BottomSheetDialog(this)
                    dialogQR.setContentView(contentViewQR)
                    dialogQR.show()
                    val qr = contentViewQR.findViewById(R.id.qr_container) as ImageView
                    val multiFormatWriter = MultiFormatWriter()
                    try {
                        val json = JSONObject()
                        json.put("bwcardid", card!!.id)
                        json.put("bwcardusid", card!!.userId)
                        val bitMatrix: BitMatrix = multiFormatWriter.encode(json.toString(), BarcodeFormat.QR_CODE, 200, 200)
                        val barEncoder = BarcodeEncoder()
                        val bitMap = barEncoder.createBitmap(bitMatrix)
                        qr.setImageBitmap(bitMap)
                    } catch (e: WriterException) {
                        e.printStackTrace()
                    }
                }
                bluetooth.setOnClickListener {
                    if (mBluetoothAdapter == null || !(mBluetoothAdapter!!.isEnabled)) {
                        val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
                        startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT)
                        dialog.dismiss()
                    } else {
                        val intent = Intent(this, BluetoothShareActivity::class.java)
                        intent.putExtra(BluetoothShareActivity.KEY_CARD, card)
                        startActivity(intent)
                        dialog.dismiss()
                    }
                }
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun setCardInfo(card: Card) {
        if (card.userImageUrl != null) {
            val requestOptions = RequestOptions()
            requestOptions.placeholder(R.drawable.card_img_empty_user)
            Glide.with(this).applyDefaultRequestOptions(requestOptions).load(card.userImageUrl).into(CardDetailPhoto)
        }
        if (card.name != null) {
            CardDetailName.text = card.name
        }
        if (card.position != null) {
            CardDetailPosition.text = card.position
        }
        if (card.company != null) {
            CardDetailCompany.text = card.companyFormmated
        }
        if (card.phone != null) {
            CardDetailPhoneLL.visibility = View.VISIBLE
            CardDetailPhone.text = card.phone
            CardDetailPhoneLL?.setOnClickListener {
                val intent = Intent(Intent.ACTION_DIAL, Uri.parse("tel:" + card.phone))
                startActivity(intent)
            }
        }
        if (card.email != null) {
            CardDetailEmailLL.visibility = View.VISIBLE
            CardDetailEmail.text = card.email
            CardDetailEmailLL?.setOnClickListener {
                val emailIntent = Intent(Intent.ACTION_SENDTO)
                emailIntent.data = Uri.parse("mailto:" + card.email)
                startActivity(Intent.createChooser(emailIntent, getString(R.string.choose_app)))
            }
        }
        if (card.notes != null) {
            CardDetailNotesLL.visibility = View.VISIBLE
            CardDetailNotes.text = card.notes
        }
        if (card.web != null) {
            CardDetailWebLL.visibility = View.VISIBLE
            CardDetailWeb.text = card.web
        }
        if (card.captionImageUrl != null) {
            CardDetailCaptionLL.visibility = View.VISIBLE
            val requestOptions = RequestOptions()
            requestOptions.placeholder(R.drawable.card_img_empty_bcard)
            Glide.with(this).applyDefaultRequestOptions(requestOptions).load(card.captionImageUrl).into(CardDetailCaption)
        }
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@CardDetailActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Adding the card to your contacts")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun setError(message: String) {
        Snackbar.make(this.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun onCardAdded() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your card has been added")
                .setPositiveButton("Accept", { _, _ ->

                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    fun sendToEdit(card: Card) {
        val intent = Intent(this, EditCardActivity::class.java)
        intent.putExtra(EditCardActivity.KEY_CARD, card)
        startActivityForResult(intent, 100)
        overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 100) {
            if (resultCode == Activity.RESULT_OK) {
                if (data != null) {
                    val cardMap: MutableMap<String, Any> = data.getSerializableExtra("result") as MutableMap<String, Any>
                    if (cardMap.containsKey("name")) card?.name = cardMap["name"].toString()
                    if (cardMap.containsKey("company")) {
                        card?.company = cardMap["company"].toString()
                        card?.companyFormmated = "@" + cardMap["company"].toString()
                    }
                    if (cardMap.containsKey("position")) card?.position = cardMap["position"].toString()
                    if (cardMap.containsKey("email")) card?.email = cardMap["email"].toString()
                    if (cardMap.containsKey("phone")) card?.phone = cardMap["phone"].toString()
                    if (cardMap.containsKey("website")) card?.web = cardMap["website"].toString()
                    if (cardMap.containsKey("notes")) card?.notes = cardMap["notes"].toString()
                    if (cardMap.containsKey("user_image_url")) card?.userImageUrl = cardMap["user_image_url"].toString()
                    if (cardMap.containsKey("caption_url")) card?.captionImageUrl = cardMap["caption_url"].toString()
                    if (card != null) {
                        setCardInfo(card!!)
                    }
                }
            }
        }
        if (requestCode == CardDetailActivity.REQUEST_ENABLE_BT) {
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                val intent = Intent(this, BluetoothShareActivity::class.java)
                intent.putExtra(BluetoothShareActivity.KEY_CARD, card)
                startActivity(intent)
            } else {
                // User did not enable Bluetooth or an error occurred
                Toast.makeText(this, "You must enable bluetooth to share and receive cards",
                        Toast.LENGTH_SHORT).show()
            }
        }
    }//onActivityResult

    companion object {
        const val KEY_CARD = "key_card"
        const val KEY_CARD_FROM = "key_card_from"
        const val REQUEST_ENABLE_BT: Int = 10
    }
}
