package com.dev.lavin.bcard.main.cards.edit

import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.graphics.PorterDuff
import android.graphics.PorterDuffColorFilter
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v4.content.ContextCompat
import android.support.v7.app.AlertDialog
import android.text.Editable
import android.text.TextWatcher
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.utils.FloatingButton
import com.dev.lavin.domain2.model.Card
import kotlinx.android.synthetic.main.activity_edit_card.*
import java.io.FileNotFoundException
import java.io.Serializable
import javax.inject.Inject


class EditCardActivity : BaseActivity<ActivityComponent>(), EditCardView, FloatingButton.IEvents {

    private var imageProfileUri: Uri? = null

    private var IMAGE_PROFILE_STATUS = false

    private var imageCaptionUri: Uri? = null

    private var IMAGE_CAPTION_STATUS = false

    private var progressDialog: ProgressDialog? = null

    var card: Card? = null

    private var currentName: String? = null
    private var currentCompany: String? = null
    private var currentPosition: String? = null
    private var currentEmail: String? = null
    private var currentPhone: String? = null
    private var currentWeb: String? = null
    private var currentNotes: String? = null

    private var currentNameModified: Boolean = false
    private var currentCompanyModified: Boolean = false
    private var currentPositionModified: Boolean = false
    private var currentEmailModified: Boolean = false
    private var currentPhonenModified: Boolean = false
    private var currentWebModified: Boolean = false
    private var currentNotesModified: Boolean = false
    private var currentCaptionModified: Boolean = false
    private var currentImageModified: Boolean = false

    @Inject
    lateinit var presenter: EditCardPresenter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_edit_card)
        setUp()
        card = intent?.extras?.getSerializable(EditCardActivity.KEY_CARD) as Card?
        if (card != null) {
            setUpCurrentValues(card!!)
            modifyListeners()
        } else {
            val returnIntent = Intent()
            setResult(Activity.RESULT_CANCELED, returnIntent)
            finish()
        }
    }

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun setUp() {
        setSupportActionBar(CardEditToolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        mSaveBtnEC?.setupScrollListeners(mAppBarEC, mNestedEC)
        mSaveBtnEC?.setButtonHideEnable(true)
        mSaveBtnEC?.setButtonEnabled(false)
        mSaveBtnEC?.setEvents(this)
        mImageEC.setOnClickListener { BringProfileImagePicker() }
        mCaptionEC.setOnClickListener { BringCaptionImagePicker() }
        val color = ContextCompat.getColor(this, R.color.primary_color_transparent)
        val cf = PorterDuffColorFilter(color, PorterDuff.Mode.MULTIPLY)
        mImageEC?.colorFilter = cf
    }

    override fun setUpCurrentValues(card: Card) {
        mNameEdtEC?.setText(card.name)
        currentName = card.name
        mCompanyEdtEC?.setText(card.company)
        currentCompany = card.company
        mPositionEdtEC?.setText(card.position)
        currentPosition = card.position
        mEmailEdtEC?.setText(card.email)
        currentEmail = card.email
        mPhoneEdtEC?.setText(card.phone)
        currentPhone = card.phone
        mWebEdtEC?.setText(card.web)
        currentWeb = card.web
        mNotesEdtEC?.setText(card.notes)
        currentNotes = card.notes
        val requestOptions = RequestOptions()
        requestOptions.placeholder(R.drawable.card_img_empty_user)
        Glide.with(this)
                .applyDefaultRequestOptions(requestOptions)
                .load(card.userImageUrl)
                .into(mImageEC)
        val requestOptionsCaption = RequestOptions()
        requestOptionsCaption.placeholder(R.drawable.card_img_empty_bcard)
        Glide.with(this)
                .applyDefaultRequestOptions(requestOptionsCaption)
                .load(card.captionImageUrl)
                .into(mCaptionEC)
    }

    override fun onButtonClick() {
        val editCardName = mNameEdtEC?.text.toString()
        val editCardCompany = mCompanyEdtEC?.text.toString()
        val editCardPosition = mPositionEdtEC?.text.toString()
        val editCardEmail = mEmailEdtEC?.text.toString()
        val editCardPhone = mPhoneEdtEC?.text.toString()
        val editCardWeb = mWebEdtEC?.text.toString()
        val editCardNotes = mNotesEdtEC?.text.toString()
        presenter.editCard(card!!.id!!, editCardName, editCardCompany, editCardPosition, editCardEmail, editCardPhone, editCardWeb, editCardNotes, imageProfileUri, imageCaptionUri)
    }

    override fun setErrorService(message: String) {
        Snackbar.make(findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@EditCardActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Editing your card")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }

    override fun showInvalidFieldsError() {
        Snackbar.make(this.findViewById(android.R.id.content), "Fields must not be empty", Snackbar.LENGTH_SHORT).show()
    }

    override fun onCardEdited(eventMap: MutableMap<String, Any>) {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your card has been modified")
                .setPositiveButton("Accept", { _, _ ->
                    val returnIntent = Intent()
                    returnIntent.putExtra("result", eventMap as Serializable)
                    setResult(Activity.RESULT_OK, returnIntent)
                    finish()
                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    private fun BringProfileImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 1000)
    }

    private fun BringCaptionImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 2000)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageProfileUri = data.data//Getting uri of the data
                val imageStream = contentResolver.openInputStream(imageProfileUri)//creating an imputstream
                val profilePicture = BitmapFactory.decodeStream(imageStream)
                mImageEC?.setImageBitmap(profilePicture)
                currentImageModified = true
                checkEnabled()
                IMAGE_PROFILE_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }
        }
        if (requestCode == 2000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageCaptionUri = data.data//Getting uri of the data
                val imageStream = contentResolver.openInputStream(imageCaptionUri)//creating an imputstream
                val captionPicture = BitmapFactory.decodeStream(imageStream)
                val requestOptionsCaption = RequestOptions()
                requestOptionsCaption.placeholder(R.drawable.card_img_empty_bcard)
                Glide.with(this)
                        .applyDefaultRequestOptions(requestOptionsCaption)
                        .load(captionPicture)
                        .into(mCaptionEC)
                currentCaptionModified = true
                checkEnabled()
                IMAGE_CAPTION_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }
        }
    }


    private fun modifyListeners() {
        mNameEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentNameModified = !currentName.equals(s.toString())
                checkEnabled()
            }
        })

        mCompanyEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentCompanyModified = !currentCompany.equals(s.toString())
                checkEnabled()
            }
        })

        mPositionEdtEC.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentPositionModified = !currentPosition.equals(s.toString())
                checkEnabled()
            }
        })

        mEmailEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentEmailModified = !currentEmail.equals(s.toString())
                checkEnabled()
            }
        })

        mPhoneEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentPhonenModified = !currentPhone.equals(s.toString())
                checkEnabled()
            }
        })

        mWebEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentWebModified = !currentWeb.equals(s.toString())
                checkEnabled()
            }
        })

        mNotesEdtEC?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentNotesModified = !currentNotes.equals(s.toString())
                checkEnabled()
            }
        })
    }

    fun checkEnabled() {
        if (currentNameModified || currentCompanyModified || currentPositionModified || currentEmailModified || currentPhonenModified || currentWebModified || currentNotesModified || currentCaptionModified || currentImageModified) {
            mSaveBtnEC?.setButtonEnabled(true)
        } else {
            mSaveBtnEC?.setButtonEnabled(false)
        }
    }

    companion object {
        const val TAG = "EditCardDetail"
        const val KEY_CARD = "key_card"
    }
}
