package com.dev.lavin.bcard.main.cards.edit

import android.net.Uri
import android.text.TextUtils
import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class EditCardPresenterImpl(editCardView: EditCardView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<EditCardView>(editCardView), EditCardPresenter, CardsInteractor.CardEditInteractorResponse {

    @Inject
    lateinit var cardsInteractor: CardsInteractor

    init {
        view = editCardView
        basePresenterComponent.injectPresenter(this)
    }

    override fun editCard(id: String, name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?) {
        if (!TextUtils.isEmpty(name) && !TextUtils.isEmpty(company) && !TextUtils.isEmpty(position) && !TextUtils.isEmpty(email) && !TextUtils.isEmpty(phone)) {
            view?.hideKeyboard()
            view?.showLoading()
            cardsInteractor.editCard(id, name, company, position, email, phone, web, notes, profile, caption, this)
        } else {
            view?.showInvalidFieldsError()
        }
    }

    override fun onCardEditInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.setErrorService(message)
    }

    override fun onCardEditInteractorResponseSuccess(cardMap: MutableMap<String, Any>) {
        view?.hideLoading()
        view?.onCardEdited(cardMap)
    }

}