package com.dev.lavin.bcard.main.contacts

import android.app.Activity
import android.app.ProgressDialog
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.CardsContactsAdapter
import com.dev.lavin.bcard.base.BaseFragment
import com.dev.lavin.bcard.blueetooth.BluetoothShareActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.fragment.BaseFragmentModule
import com.dev.lavin.bcard.main.cards.detail.CardDetailActivity
import com.dev.lavin.bcard.utils.InternetUtils
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.Contacts
import com.google.firebase.auth.FirebaseAuth
import com.google.zxing.integration.android.IntentIntegrator
import kotlinx.android.synthetic.main.fragment_contacts.*
import org.json.JSONException
import org.json.JSONObject
import javax.inject.Inject


class ContactsFragment : BaseFragment(), ContactsView, CardsContactsAdapter.OnCardSelectedListener {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var contacts: Contacts

    @Inject
    lateinit var presenter: ContactsPresenter

    private var cardsAdapter: CardsContactsAdapter? = null

    private var mBluetoothAdapter: BluetoothAdapter? = null

    //qr code scanner object
    private var qrScan: IntentIntegrator? = null

    private var progressDialog: ProgressDialog? = null

    private var cardSelected: Card? = null

    override val xmlToInflate: Int
        get() = R.layout.fragment_contacts

    override fun onBackPressed() {

    }

    override fun initDIComponent() {
        fragmentComponent = getParentComponent(ActivityComponent::class.java).with(BaseFragmentModule(this, activity!!))
        fragmentComponent!!.inject(this)
    }

    override fun initViews() {
        setHasOptionsMenu(true)
        //intializing scan object
        qrScan = IntentIntegrator.forSupportFragment(this)
        qrScan?.setOrientationLocked(false)
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
        if (mAuth.currentUser != null) {
            presenter.getContactsCards(mAuth.currentUser!!.uid)
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        inflater!!.inflate(R.menu.contacts_menu, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        return when (item!!.itemId) {
            R.id.action_contact_add -> {
                qrScan?.initiateScan()
                true
            }
            R.id.action_bluetooth_visible -> {
                ensureDiscoverable()
                true
            }
            else -> false
        }
    }

    override fun setCardList(cardList: MutableList<Card>) {
        contacts.addContacts(cardList)
        cardsAdapter = CardsContactsAdapter(cardList!!, this)
        ContactsList?.layoutManager = LinearLayoutManager(context)
        ContactsList?.adapter = cardsAdapter
        ContactsList?.setHasFixedSize(true)
        ContactsEmpty?.visibility = View.GONE
        ContactsList?.visibility = View.VISIBLE
        cardsAdapter?.notifyDataSetChanged()
    }

    override fun setEmptyContacts() {
        ContactsEmpty?.visibility = View.VISIBLE
        ContactsList?.visibility = View.GONE
    }

    override fun hideLoading() {
        ContactsProgress?.visibility = View.GONE
    }

    override fun showLoading() {
        ContactsProgress?.visibility = View.VISIBLE
    }

    override fun setConnectionError() {
        Toast.makeText(context, "Check your connectivity", Toast.LENGTH_LONG).show()
    }

    override fun showError(message: String) {
        if (context != null) {
            Toast.makeText(context, message, Toast.LENGTH_LONG).show()
        }
    }

    fun sendToDetail(card: Card) {
        val intent = Intent(context, CardDetailActivity::class.java)
        intent.putExtra(CardDetailActivity.KEY_CARD, card)
        intent.putExtra(CardDetailActivity.KEY_CARD_FROM, false)
        startActivity(intent)
        activity?.overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    override fun onCardSelected(card: Card) {
        cardSelected = card
        sendToDetail(card)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        val result = IntentIntegrator.parseActivityResult(requestCode, resultCode, data)
        if (result != null) {
            //if qrcode has nothing in it
            if (result.contents == null) {
                Toast.makeText(context, "Result Not Found", Toast.LENGTH_LONG).show()
            } else {
                try {
                    //converting the data to json
                    val obj = JSONObject(result.contents)
                    //setting values to textviews
                    //Toast.makeText(context, obj.getString("bwcardid") + obj.getString("bwcardusid"), Toast.LENGTH_LONG).show()
                    if (obj.has("bwcardid") && obj.has("bwcardusid")) {
                        if (obj.getString("bwcardusid") != mAuth.currentUser!!.uid) {
                            if (contacts != null && !contacts.checkIfAlreadyContacts(obj.getString("bwcardid"))) {
                                if (InternetUtils.checkInternetConnection(context!!)) {
                                    presenter.addCardToContacts(obj.getString("bwcardid"), mAuth.currentUser!!.uid)
                                } else {
                                    setConnectionError()
                                }
                            } else {
                                Toast.makeText(context, "The card is already added in your contacts", Toast.LENGTH_LONG).show()
                            }
                        } else {
                            Toast.makeText(context, "You are trying to add your own card", Toast.LENGTH_LONG).show()
                        }
                    } else {
                        Toast.makeText(context, "Incorrect data", Toast.LENGTH_LONG).show()
                    }
                } catch (e: JSONException) {
                    e.printStackTrace()
                    Toast.makeText(context, result.contents, Toast.LENGTH_LONG).show()
                }
            }
        } else if (requestCode == ContactsFragment.REQUEST_ENABLE_BT) {
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                val intent = Intent(activity, BluetoothShareActivity::class.java)
                intent.putExtra(BluetoothShareActivity.KEY_CARD, cardSelected)
                startActivity(intent)
            } else {
                // User did not enable Bluetooth or an error occurred
                Toast.makeText(activity, "You must enable bluetooth to share and receive cards",
                        Toast.LENGTH_SHORT).show()
            }
        } else if (requestCode == ContactsFragment.REQUEST_VISIBLE_BT) {
            // When the request to visibility Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                //Toast.makeText(activity, "Your device is in discovery mode",
                //      Toast.LENGTH_SHORT).show()
                ensureDiscoverable()
            } else {
                // User did not enable Bluetooth visibility or an error occurred
                Toast.makeText(activity, "This can cause wrong discovery of your device",
                        Toast.LENGTH_SHORT).show()
            }
        }
    }

    override fun showAddLoading() {
        progressDialog = ProgressDialog(activity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Adding the card to your contacts")
        progressDialog?.show()
    }

    override fun hideAddLoading() {
        progressDialog?.dismiss()
    }

    override fun onCardAdded() {
        val builder = AlertDialog.Builder(context!!)
        builder.setCancelable(false)
        builder.setMessage("The card has been added")
                .setPositiveButton("Accept", { _, _ ->

                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    private fun ensureDiscoverable() {
        if (mBluetoothAdapter == null || !(mBluetoothAdapter!!.isEnabled)) {
            val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
            startActivityForResult(enableBtIntent, ContactsFragment.REQUEST_VISIBLE_BT)
        } else if (mBluetoothAdapter != null && mBluetoothAdapter!!.scanMode != BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
            val discoverableIntent = Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE)
            discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300)
            startActivity(discoverableIntent)
        } else if (mBluetoothAdapter != null && mBluetoothAdapter!!.scanMode == BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
            Toast.makeText(activity, "Your device is already in discovery mode",
                    Toast.LENGTH_SHORT).show()
        }
    }

    companion object {
        const val REQUEST_ENABLE_BT: Int = 11
        const val REQUEST_VISIBLE_BT: Int = 20
        fun newInstance(): ContactsFragment {
            return ContactsFragment()
        }
    }
}
