package com.dev.lavin.bcard.main.contacts

import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.bcard.utils.CardUtils
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import com.dev.lavin.domain2.model.Card
import java.util.*
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class ContactsPresenterImpl(homeView: ContactsView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<ContactsView>(homeView), ContactsPresenter, CardsInteractor.CardsContactInteractorResponse, CardsInteractor.CardsContactsAddInteractorResponse {

    @Inject
    lateinit var cardsInteractor: CardsInteractor

    private var cardListHomeFragment: MutableList<Card>? = ArrayList()

    init {
        view = homeView
        basePresenterComponent.injectPresenter(this)
    }

    override fun getContactsCards(id: String) {
        view?.showLoading()
        cardsInteractor.getContactCards(id, this)
    }

    override fun onCardsContactInteractorResponseSuccess(cardList: MutableMap<String, MutableList<Card>>) {
        view?.hideLoading()
        CardUtils.addCards(cardList, cardListHomeFragment)
        CardUtils.modifyCards(cardList, cardListHomeFragment)
        if (cardListHomeFragment!!.size > 0) {
            view?.setCardList(cardListHomeFragment!!)
        } else {
            view?.setEmptyContacts()
        }
    }

    override fun onCardsContactInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.showError(message)
    }

    override fun addCardToContacts(idCard: String?, idUser: String?) {
        if (idCard != null && idUser != null) {
            view?.showAddLoading()
            cardsInteractor.addCardToContacts(idCard, idUser, this)
        }
    }

    override fun onCardsContactsAddInteractorResponseSuccess() {
        view?.hideAddLoading()
        view?.onCardAdded()
    }

    override fun onCardsContactsAddInteractorResponseFailure(message: String) {
        view?.hideAddLoading()
        view?.showError(message)
    }
}