package com.dev.lavin.bcard.main.events

import android.content.Intent
import android.support.v7.widget.LinearLayoutManager
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.EventsAdapter
import com.dev.lavin.bcard.base.BaseFragment
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.fragment.BaseFragmentModule
import com.dev.lavin.bcard.main.events.create.NewEventActivity
import com.dev.lavin.bcard.main.events.detail.EventDetailActivity
import com.dev.lavin.domain2.model.Event
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.FirebaseFirestore
import kotlinx.android.synthetic.main.fragment_events.*
import javax.inject.Inject

class EventsFragment : BaseFragment(), EventsView, EventsAdapter.OnEventSelectedListener {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var mFireStore: FirebaseFirestore

    @Inject
    lateinit var presenter: EventsPresenter

    private var eventsAdapter: EventsAdapter? = null

    override val xmlToInflate: Int
        get() = R.layout.fragment_events

    override fun onBackPressed() {

    }

    override fun initDIComponent() {
        fragmentComponent = getParentComponent(ActivityComponent::class.java).with(BaseFragmentModule(this, activity!!))
        fragmentComponent!!.inject(this)
    }

    override fun initViews() {
        setHasOptionsMenu(true)
        // Initialize Firestore and the main RecyclerView
        if (mAuth.currentUser != null) {
            presenter.getEvents()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        inflater!!.inflate(R.menu.events_menu, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        return when (item!!.itemId) {
            R.id.action_event_add_btn -> {
                sendToNewEvent()
                true
            }
            else -> false
        }
    }

    override fun setEventsList(eventsList: MutableList<Event>) {
        eventsAdapter = EventsAdapter(eventsList!!, this, mFireStore)
        EventsList?.layoutManager = LinearLayoutManager(context)
        EventsList?.adapter = eventsAdapter
        EventsList?.setHasFixedSize(true)
        EventsEmpty?.visibility = View.GONE
        EventsList?.visibility = View.VISIBLE
        eventsAdapter?.notifyDataSetChanged()
    }

    override fun setEmptyEvents() {
        EventsEmpty?.visibility = View.VISIBLE
        EventsList?.visibility = View.GONE
    }

    override fun hideLoading() {
        EventsProgress?.visibility = View.GONE
    }

    override fun showLoading() {
        EventsProgress?.visibility = View.VISIBLE
    }

    override fun setConnectionError() {
        EventsIntenetError?.visibility = View.VISIBLE
    }

    override fun showError(message: String) {
        /*if(view != null) {
            Snackbar.make(view!!.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
        }*/
        if (context != null) {
            Toast.makeText(context, message, Toast.LENGTH_LONG).show()
        }
    }

    override fun onEventSelected(event: Event) {
        sendToDetail(event)
    }

    fun sendToNewEvent() {
        val intent = Intent(context, NewEventActivity::class.java)
        startActivity(intent)
        activity?.overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    fun sendToDetail(event: Event) {
        val intent = Intent(context, EventDetailActivity::class.java)
        intent.putExtra(EventDetailActivity.KEY_EVENT_ID, event.id)
        startActivity(intent)
    }

    companion object {

        fun newInstance(): EventsFragment {
            return EventsFragment()
        }
    }
}
