package com.dev.lavin.bcard.main.events

import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.bcard.utils.EventsUtils
import com.dev.lavin.domain2.interactors.events.EventsInteractor
import com.dev.lavin.domain2.model.Event
import java.util.*
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class EventsPresenterImpl(eventsView: EventsView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<EventsView>(eventsView), EventsPresenter, EventsInteractor.EventsInteractorResponse {


    @Inject
    lateinit var eventsInteractor: EventsInteractor

    private var cardListEventsFragment: MutableList<Event>? = ArrayList()

    init {
        view = eventsView
        basePresenterComponent.injectPresenter(this)
    }

    override fun getEvents() {
        view?.showLoading()
        eventsInteractor.getEvents(this)
    }

    override fun onEventsInteractorResponseSuccess(eventList: MutableMap<String, MutableList<Event>>) {
        view?.hideLoading()
        EventsUtils.addCards(eventList, cardListEventsFragment)
        EventsUtils.modifyCards(eventList, cardListEventsFragment)
        view?.setEventsList(cardListEventsFragment!!)
    }

    override fun onEventsEmptyInteractorResponse() {
        view?.hideLoading()
        view?.setEmptyEvents()
    }

    override fun onEventsInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.showError(message)
    }

}