package com.dev.lavin.bcard.main.events.cardadd

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.view.MenuItem
import android.view.View
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.CardListAdapter
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.utils.FloatingButton
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.Event
import com.google.firebase.auth.FirebaseAuth
import kotlinx.android.synthetic.main.activity_event_card_add.*
import javax.inject.Inject


class EventCardAddActivity : BaseActivity<ActivityComponent>(), EventCardAddView, FloatingButton.IEvents {

    private var adapter: CardListAdapter? = null

    private var progressDialog: ProgressDialog? = null

    private var event: Event? = null

    @Inject
    lateinit var presenter: EventCardAddPresenter

    @Inject
    lateinit var mAuth: FirebaseAuth

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_event_card_add)
        setUp()
        event = intent.extras?.getSerializable(EventCardAddActivity.KEY_EVENT_ID) as Event
        if (event != null) {
            if (mAuth.currentUser != null) {
                presenter.getUserCards(mAuth.currentUser!!.uid)
                mSaveBtnecA?.setupScrollListeners(mAppBarecA, mNestedecA)
                mSaveBtnecA?.setButtonHideEnable(true)
                mSaveBtnecA?.setButtonEnabled(true)
                mSaveBtnecA?.setEvents(this)
            }
        } else {
            finish()
        }
    }

    override fun onButtonClick() {
        if (adapter != null && adapter!!.selectedItem != null) {
            if (!event!!.cards!!.containsKey(adapter!!.selectedItem!!.id!!)) {
                presenter.addCard(event!!.id!!, adapter!!.selectedItem!!.id!!)
            } else {
                Snackbar.make(findViewById(android.R.id.content), "Card already added", Snackbar.LENGTH_SHORT).show()
            }
        } else {
            Snackbar.make(findViewById(android.R.id.content), "You must select one item", Snackbar.LENGTH_SHORT).show()
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun setUp() {
        setSupportActionBar(NewCardAddedToolbar)
        supportActionBar?.title = "Select card"
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setDisplayShowHomeEnabled(true)
    }

    /*override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.events_select_card_menu, menu)
        return true
    }*/

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                val returnIntent = Intent()
                setResult(Activity.RESULT_CANCELED, returnIntent)
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    private fun populateRecyclerView(cardList: MutableList<Card>) {
        adapter = CardListAdapter(this, cardList)
        CardListSelection?.layoutManager = LinearLayoutManager(this)
        CardListSelection?.adapter = adapter
        CardListSelection?.setHasFixedSize(true)
        adapter?.notifyDataSetChanged()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun onCardAdded() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your card has been added")
                .setPositiveButton("Accept", { _, _ ->
                    finish()
                    overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@EventCardAddActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Adding your card to the Event")
        progressDialog?.show()
    }

    override fun setErrorService(message: String) {
        Snackbar.make(findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun setCardEventList(cardList: MutableList<Card>) {
        mSaveBtnecA?.setButtonEnabled(true)
        NewCardAddeeEmpty?.visibility = View.GONE
        populateRecyclerView(cardList)
    }

    override fun setEmpty() {
        mSaveBtnecA?.setButtonEnabled(false)
        NewCardAddeeEmpty?.visibility = View.VISIBLE
    }

    companion object {
        const val KEY_EVENT_ID = "key_event_id"
    }
}
