package com.dev.lavin.bcard.main.events.create

import android.annotation.SuppressLint
import android.app.Activity
import android.app.DatePickerDialog
import android.app.ProgressDialog
import android.app.TimePickerDialog
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v7.app.AlertDialog
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.utils.FloatingButton
import kotlinx.android.synthetic.main.activity_new_event.*
import java.io.FileNotFoundException
import java.text.SimpleDateFormat
import java.util.*
import javax.inject.Inject


class NewEventActivity : BaseActivity<ActivityComponent>(), NewEventView, FloatingButton.IEvents {

    lateinit var myCalendar: Calendar

    private var imageUri: Uri? = null

    private var IMAGE_STATUS = false

    private var progressDialog: ProgressDialog? = null

    var newEventDate: Long? = 0L

    @Inject
    lateinit var presenter: NewEventPresenter

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    @SuppressLint("SetTextI18n")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_new_event)
        myCalendar = Calendar.getInstance()
        setUp()
        val date = DatePickerDialog.OnDateSetListener { _, year, monthOfYear, dayOfMonth ->
            myCalendar.set(Calendar.YEAR, year)
            myCalendar.set(Calendar.MONTH, monthOfYear)
            myCalendar.set(Calendar.DAY_OF_MONTH, dayOfMonth)
            updateLabel()
        }

        NewEventDate.setOnClickListener {
            DatePickerDialog(this@NewEventActivity, date, myCalendar
                    .get(Calendar.YEAR), myCalendar.get(Calendar.MONTH),
                    myCalendar.get(Calendar.DAY_OF_MONTH)).show()
        }

        NewEventTime.setOnClickListener {
            // TODO Auto-generated method stub
            val mcurrentTime = Calendar.getInstance()
            val hour = mcurrentTime.get(Calendar.HOUR_OF_DAY)
            val minute = mcurrentTime.get(Calendar.MINUTE)

            val mTimePicker: TimePickerDialog
            mTimePicker = TimePickerDialog(this@NewEventActivity, TimePickerDialog.OnTimeSetListener { _, selectedHour, selectedMinute ->
                val minutes: String = if (selectedMinute < 10) {
                    "0$selectedMinute"
                } else {
                    selectedMinute.toString()
                }
                NewEventTime.setText(selectedHour.toString() + ":" + minutes)
            }, hour, minute, true)
            mTimePicker.setTitle("Select Time")
            mTimePicker.show()
        }

        NewEventImage.setOnClickListener { BringImagePicker() }
        mSaveBtnCE?.setupScrollListeners(mAppBarNe, mNestedNe)
        mSaveBtnCE?.setButtonHideEnable(true)
        mSaveBtnCE?.setButtonEnabled(true)
        mSaveBtnCE?.setEvents(this)
    }

    override fun onButtonClick() {
        val newEventName = NewEventName?.text.toString()
        val newEventLocation = NewEventLocation?.text.toString()
        val newEventCategory = NewEventCategory?.text.toString()
        val newEventDescription = NewEventDescription?.text.toString()
        val newEventTime = NewEventTime?.text.toString()
        presenter.createEvent(newEventName, newEventDate!!, newEventTime, newEventLocation, newEventCategory, newEventDescription, imageUri)
    }

    override fun onBackPressed() {
        super.onBackPressed()
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun setUp() {
        setSupportActionBar(NewEventToolbar)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setDisplayShowHomeEnabled(true)
    }

    private fun updateLabel() {
        val myFormat = "dd/MM/yy" //In which you need put here
        val sdf = SimpleDateFormat(myFormat, Locale.FRENCH)
        NewEventDate?.setText(sdf.format(myCalendar.time))
        newEventDate = myCalendar.time.time
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }

    override fun onEventCreated() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your event has been created")
                .setPositiveButton("Accept", { _, _ ->
                    finish()
                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@NewEventActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Creating Your Event")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun showServiceError(message: String) {
        Snackbar.make(this.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun showInvalidFieldsError() {
        Snackbar.make(this.findViewById(android.R.id.content), "Fields must not be empty", Snackbar.LENGTH_SHORT).show()
    }

    private fun BringImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 1000)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageUri = data.data//Getting uri of the data
                val imageStream = contentResolver.openInputStream(imageUri)//creating an imputstream
                val profilePicture = BitmapFactory.decodeStream(imageStream)
                NewEventImage?.setImageBitmap(profilePicture)
                IMAGE_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }

        }
    }

}
