package com.dev.lavin.bcard.main.events.create

import android.net.Uri
import android.text.TextUtils
import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.domain2.interactors.events.EventsInteractor
import javax.inject.Inject


/**
 * Created by lavin
 * on 16/04/2018.
 */
class NewEventPresenterImpl(newEventView: NewEventView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<NewEventView>(newEventView), NewEventPresenter, EventsInteractor.EventsCreateInteractorResponse {

    @Inject
    lateinit var eventsInteractor: EventsInteractor

    init {
        view = newEventView
        basePresenterComponent.injectPresenter(this)
    }

    override fun createEvent(name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri?) {
        if (!TextUtils.isEmpty(name) && date != 0L && !TextUtils.isEmpty(time) && !TextUtils.isEmpty(location) && !TextUtils.isEmpty(category) && !TextUtils.isEmpty(description) && url != null) {
            view?.hideKeyboard()
            view?.showLoading()
            eventsInteractor.createEvent(name, date, time, location, category, description, url, this)
        } else {
            view?.showInvalidFieldsError()
        }
    }

    override fun onEventsCreateInteractorResponseSuccess() {
        view?.hideLoading()
        view.onEventCreated()
    }

    override fun onEventsCreateInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.showServiceError(message)
    }

}