package com.dev.lavin.bcard.main.events.detail

import android.app.Activity
import android.app.ProgressDialog
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v4.app.ShareCompat
import android.support.v7.app.ActionBar
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.text.format.DateFormat
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.CardsEventAdapter
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.main.cards.detail.CardDetailActivity
import com.dev.lavin.bcard.main.events.cardadd.EventCardAddActivity
import com.dev.lavin.bcard.main.events.edit.EditEventActivity
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.Contacts
import com.dev.lavin.domain2.model.Event
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.dynamiclinks.DynamicLink
import com.google.firebase.dynamiclinks.FirebaseDynamicLinks
import kotlinx.android.synthetic.main.activity_event_detail.*
import kotlinx.android.synthetic.main.activity_event_detail_data_row.*
import kotlinx.android.synthetic.main.content_event_detail.*
import java.net.URL
import java.net.URLDecoder
import java.util.*
import javax.inject.Inject
import kotlin.collections.HashMap


class EventDetailActivity : BaseActivity<ActivityComponent>(), EventDetailView, CardsEventAdapter.OnCardEventSelectedListener {

    @Inject
    lateinit var presenter: EventDetailPresenter

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var contacts: Contacts

    private var cardsAdapter: CardsEventAdapter? = null

    private var currentEvent: Event? = null

    private var eventId: String? = null

    private var progressDialog: ProgressDialog? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_event_detail)
        setSupportActionBar(EventDetailToolbar)
        val actionBar: ActionBar? = supportActionBar
        actionBar?.setDisplayHomeAsUpEnabled(true)
        EventDetailCollapsingToolbar?.title = ""
        // Get event ID from extras
        eventId = intent.extras?.getString(KEY_EVENT_ID)
        if (eventId != null) {
            showLoading()
            presenter.getCardsFromEvent(eventId!!)
            presenter.getEventDetailInfo(eventId!!)
            EventDetailFab.setOnClickListener { view ->
                val intent = Intent(this, EventCardAddActivity::class.java)
                intent.putExtra(EventCardAddActivity.KEY_EVENT_ID, currentEvent)
                startActivity(intent)
                overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)

            }
        } else {
            setEmptyView()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.events_detail_menu, menu)
        if (currentEvent != null && mAuth.currentUser != null && currentEvent!!.user_id == mAuth.currentUser!!.uid) {
            val item = menu.findItem(R.id.action_event_edit_btn)
            item?.isVisible = true
        }
        return true
    }

    override fun setEmptyView() {
        EventDetailCoordinator?.visibility = View.VISIBLE
        EventDetailEmpty?.visibility = View.VISIBLE
        EvenDetailListContainer?.visibility = View.GONE
        EventDetailFab?.visibility = View.GONE
    }

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                finish()
                true
            }
            R.id.action_event_share_btn -> {
                try {
                    val dynamicLink = FirebaseDynamicLinks.getInstance().createDynamicLink()
                            .setLink(Uri.parse("http://bwcards.com/?id=" + currentEvent?.id))
                            .setDynamicLinkDomain("p5856.app.goo.gl")
                            // Open links with this app on Android
                            .setAndroidParameters(DynamicLink.AndroidParameters.Builder("com.dev.lavin.bcard").build())
                            .buildDynamicLink()

                    val dynamicLinkUri = dynamicLink.uri
                    val url = URL(URLDecoder.decode(dynamicLinkUri.toString(),
                            "UTF-8"))

                    startActivity(Intent.createChooser(ShareCompat.IntentBuilder.from(this)
                            .setType("text/plain")
                            .setSubject("Check the amazing event:" + currentEvent?.name)
                            .setText(url.toString())
                            .intent, "Choose option"))
                } catch (e: Exception) {
                    Log.i(EventDetailActivity.TAG, "Could not decode Uri: " + e.localizedMessage)
                }
                true
            }
            R.id.action_event_edit_btn -> {
                if (currentEvent != null) {
                    sendToEdit(currentEvent!!)
                }
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun setCardList(cardList: MutableList<Card>) {
        EventDetailNoCards.visibility = View.GONE
        cardsAdapter = CardsEventAdapter(mAuth.currentUser!!.uid, cardList!!, this)
        EventDetailList?.layoutManager = LinearLayoutManager(this)
        EventDetailList?.adapter = cardsAdapter
        EventDetailList?.setHasFixedSize(true)
        cardsAdapter?.notifyDataSetChanged()
        if (cardList.size > 0) {
            actualizeCardMap(cardList)
        }
    }

    fun actualizeCardMap(cardList: MutableList<Card>) {
        val mapCards: HashMap<String, Boolean> = HashMap()
        for (card: Card in cardList) {
            mapCards[card.id!!] = true
        }
        currentEvent?.cards = mapCards
    }

    override fun setEmptyCardsView() {
        EventDetailNoCards.visibility = View.VISIBLE
    }

    override fun setEventInfo(event: Event?) {
        hideLoading()
        EventDetailCoordinator?.visibility = View.VISIBLE
        EvenDetailListContainer?.visibility = View.VISIBLE
        if (event != null) {
            invalidateOptionsMenu()
            currentEvent = event
            setValues(currentEvent)
        }
    }

    private fun setValues(event: Event?) {
        setName(event)
        setTimeStamp(event)
        setSubject(event)
        setTime(event)
        setLocation(event)
        setDescription(event)
        setImage(event)
    }

    private fun setImage(event: Event?) {
        if (event?.eventImageUrl != null) {
            val requestOptions = RequestOptions()
            requestOptions.placeholder(R.drawable.event_img_empty_card)
            Glide.with(this)
                    .applyDefaultRequestOptions(requestOptions)
                    .load(event.eventImageUrl)
                    .into(EventDetailImage)
        }
    }

    private fun setName(event: Event?) {
        if (event?.name != null) {
            EventDetailCollapsingToolbar?.title = event.name
        }
    }

    private fun setTimeStamp(event: Event?) {
        if (event?.timestamp != null) {
            val millisecond = event.timestamp!!.time
            val dateString = DateFormat.format("dd/MM/yyyy", Date(millisecond)).toString()
            EventDetailDateLayout?.visibility = View.VISIBLE
            EventDetailDate?.text = dateString
        }
    }

    private fun setSubject(event: Event?) {
        if (event?.subject != null) {
            EventDetailSubjectLayout?.visibility = View.VISIBLE
            EventDetailSubject?.text = event.subject
        }
    }

    private fun setTime(event: Event?) {
        if (event?.time != null) {
            EventDetailTimeLayout?.visibility = View.VISIBLE
            EventDetailTime?.text = event.time
        }
    }

    private fun setLocation(event: Event?) {
        if (event?.location != null) {
            EventDetailLocationLayout?.visibility = View.VISIBLE
            EventDetailLocation?.text = event.location
        }
    }

    private fun setDescription(event: Event?) {
        if (event?.description != null) {
            EventDetailNotesLayout?.visibility = View.VISIBLE
            EventDetailNotes?.text = event.description
        }
    }


    override fun hideLoading() {
        EventDetailProgress?.visibility = View.GONE
    }

    override fun showLoading() {
        EventDetailProgress?.visibility = View.VISIBLE
    }

    override fun showServiceError(message: String) {
        Snackbar.make(this.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    fun sendToDetail(card: Card) {
        val intent = Intent(this, CardDetailActivity::class.java)
        intent.putExtra(CardDetailActivity.KEY_CARD, card)
        intent.putExtra(CardDetailActivity.KEY_CARD_FROM, true)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    fun sendToEdit(currentEvent: Event) {
        val intent = Intent(this, EditEventActivity::class.java)
        intent.putExtra(EditEventActivity.KEY_EVENT, currentEvent)
        startActivityForResult(intent, 100)
        overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    override fun onCardSelected(card: Card) {
        sendToDetail(card)
    }

    override fun onAddCardSelected(card: Card) {
        if (card != null && mAuth.currentUser != null) {
            if (contacts != null && !contacts.checkIfAlreadyContacts(card.id!!)) {
                presenter.addCardToContacts(card!!.id!!, mAuth.currentUser!!.uid)
            } else {
                Toast.makeText(this, "The card is already added in your contacts", Toast.LENGTH_LONG).show()
            }
        }
    }

    override fun showAddLoading() {
        progressDialog = ProgressDialog(this@EventDetailActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Adding the card to your contacts")
        progressDialog?.show()
    }

    override fun hideAddLoading() {
        progressDialog?.dismiss()
    }

    override fun onCardAdded() {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("The card has been added")
                .setPositiveButton("Accept") { _, _ ->

                }
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 100) {
            if (resultCode == Activity.RESULT_OK) {
                if (data != null) {
                    val eventMap: MutableMap<String, Any> = data.getSerializableExtra("result") as MutableMap<String, Any>
                    if (eventMap.containsKey("name")) currentEvent?.name = eventMap["name"].toString()
                    if (eventMap.containsKey("time")) currentEvent?.time = eventMap["time"].toString()
                    if (eventMap.containsKey("timestamp")) currentEvent?.timestamp = Date(eventMap["timestamp"] as Long)
                    if (eventMap.containsKey("location")) currentEvent?.location = eventMap["location"].toString()
                    if (eventMap.containsKey("category")) currentEvent?.subject = eventMap["category"].toString()
                    if (eventMap.containsKey("description")) currentEvent?.description = eventMap["description"].toString()
                    if (eventMap.containsKey("event_image_url")) currentEvent?.eventImageUrl = eventMap["event_image_url"].toString()
                    setValues(currentEvent)
                }
            }
        }
    }//onActivityResult

    companion object {
        const val TAG = "EventDetail"
        const val KEY_EVENT_ID = "key_event_id"
    }
}
