package com.dev.lavin.bcard.main.events.detail

import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.bcard.utils.CardUtils
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import com.dev.lavin.domain2.interactors.events.EventsInteractor
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.Event
import java.util.*
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class EventsDetailPresenterImpl(eventDetailView: EventDetailView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<EventDetailView>(eventDetailView), EventDetailPresenter, CardsInteractor.CardFromEventInteractorResponse, EventsInteractor.EventDetailInteractorResponse, CardsInteractor.CardsContactsAddInteractorResponse {

    @Inject
    lateinit var eventsInteractor: EventsInteractor

    @Inject
    lateinit var cardsInteractor: CardsInteractor

    private var cardListEventDetail: MutableList<Card>? = ArrayList()

    init {
        view = eventDetailView
        basePresenterComponent.injectPresenter(this)
    }

    override fun getEventDetailInfo(id: String) {
        eventsInteractor.getEventDetail(id, this)
    }

    override fun onEventDetailInteractorResponseSuccess(event: Event) {
        view?.setEventInfo(event)
    }

    override fun onEventDetailInteractorResponseFailure(message: String) {
        view?.showServiceError(message)
    }

    override fun getCardsFromEvent(id: String) {
        cardsInteractor.getCardsFromEvent(id, this)
    }

    override fun onEventNotExistInteractorResponse() {
        view?.setEmptyView()
    }

    override fun onCardFromEventInteractorResponseSuccess(cardList: MutableMap<String, MutableList<Card>>) {
        CardUtils.addCards(cardList, cardListEventDetail)
        CardUtils.modifyCards(cardList, cardListEventDetail)
        view?.setCardList(cardListEventDetail!!)
    }

    override fun onCardFromEventInteractorResponseFailure(message: String) {
        view?.showServiceError(message)
    }

    override fun onCardsFromEventEmpty() {
        view?.setEmptyCardsView()
    }

    override fun addCardToContacts(idCard: String?, idUser: String?) {
        if (idCard != null && idUser != null) {
            view?.showAddLoading()
            cardsInteractor.addCardToContacts(idCard, idUser, this)
        }
    }

    override fun onCardsContactsAddInteractorResponseSuccess() {
        view?.hideAddLoading()
        view?.onCardAdded()
    }

    override fun onCardsContactsAddInteractorResponseFailure(message: String) {
        view?.hideAddLoading()
        view?.showServiceError(message)
    }

}