package com.dev.lavin.bcard.main.events.edit

import android.annotation.SuppressLint
import android.app.Activity
import android.app.DatePickerDialog
import android.app.ProgressDialog
import android.app.TimePickerDialog
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v7.app.AlertDialog
import android.text.Editable
import android.text.TextWatcher
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.utils.FloatingButton
import com.dev.lavin.domain2.model.Event
import kotlinx.android.synthetic.main.activity_edit_event_detail.*
import java.io.FileNotFoundException
import java.io.Serializable
import java.text.SimpleDateFormat
import java.util.*
import javax.inject.Inject


class EditEventActivity : BaseActivity<ActivityComponent>(), EditEventView, FloatingButton.IEvents {

    lateinit var myCalendar: Calendar

    private var imageUri: Uri? = null

    private var IMAGE_STATUS = false

    private var progressDialog: ProgressDialog? = null

    var event: Event? = null

    var editEventDate: Long? = 0L

    private var currentName: String? = null
    private var currentTime: String? = null
    private var currentDate: String? = null
    private var currentCategory: String? = null
    private var currentLocation: String? = null
    private var currentDescription: String? = null

    private var currentNameModified: Boolean = false
    private var currentTimeModified: Boolean = false
    private var currentDateModified: Boolean = false
    private var currentCategoryModified: Boolean = false
    private var currentLocationModified: Boolean = false
    private var currentDescriptionModified: Boolean = false
    private var currentImageModified: Boolean = false

    @Inject
    lateinit var presenter: EditEventPresenter

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    @SuppressLint("SetTextI18n")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_edit_event_detail)
        setUp()
        event = intent?.extras?.getSerializable(EditEventActivity.KEY_EVENT) as Event?
        if (event != null) {
            setUpCurrentValues(event!!)
            modifyListeners()
            myCalendar = Calendar.getInstance()
            val date = DatePickerDialog.OnDateSetListener { _, year, monthOfYear, dayOfMonth ->
                myCalendar.set(Calendar.YEAR, year)
                myCalendar.set(Calendar.MONTH, monthOfYear)
                myCalendar.set(Calendar.DAY_OF_MONTH, dayOfMonth)
                updateLabel()
            }

            EditEventDate.setOnClickListener {
                DatePickerDialog(this@EditEventActivity, date, myCalendar
                        .get(Calendar.YEAR), myCalendar.get(Calendar.MONTH),
                        myCalendar.get(Calendar.DAY_OF_MONTH)).show()
            }

            EditEventTime.setOnClickListener {
                // TODO Auto-generated method stub
                val mcurrentTime = Calendar.getInstance()
                val hour = mcurrentTime.get(Calendar.HOUR_OF_DAY)
                val minute = mcurrentTime.get(Calendar.MINUTE)

                val mTimePicker: TimePickerDialog
                mTimePicker = TimePickerDialog(this@EditEventActivity, TimePickerDialog.OnTimeSetListener { _, selectedHour, selectedMinute ->
                    val minutes: String = if (selectedMinute < 10) {
                        "0$selectedMinute"
                    } else {
                        selectedMinute.toString()
                    }
                    EditEventTime?.setText(selectedHour.toString() + ":" + minutes)
                }, hour, minute, true)
                mTimePicker.setTitle("Select Time")
                mTimePicker.show()
            }

            EditEventImage.setOnClickListener { BringImagePicker() }
            mSaveBtnEE?.setupScrollListeners(mAppBarEe, mNestedEe)
            mSaveBtnEE?.setButtonHideEnable(true)
            mSaveBtnEE?.setButtonEnabled(false)
            mSaveBtnEE?.setEvents(this)
        } else {
            val returnIntent = Intent()
            setResult(Activity.RESULT_CANCELED, returnIntent)
            finish()
        }
    }

    override fun setUpCurrentValues(event: Event) {
        val myFormat = "dd/MM/yy" //In which you need put here
        val sdf = SimpleDateFormat(myFormat, Locale.FRENCH)
        editEventDate = event.timestamp!!.time
        currentTime = event.timestamp!!.time.toString()
        EditEventName?.setText(event.name)
        currentName = event.name
        EditEventDate?.setText(sdf.format(event.timestamp))
        currentDate = sdf.format(event.timestamp)
        EditEventTime?.setText(event.time)
        EditEventLocation?.setText(event.location)
        currentLocation = event.location
        EditEventCategory?.setText(event.subject)
        currentCategory = event.subject
        EditEventDescription?.setText(event.description)
        currentDescription = event.description
        val requestOptions = RequestOptions()
        requestOptions.placeholder(R.drawable.event_img_empty_card)
        Glide.with(this)
                .applyDefaultRequestOptions(requestOptions)
                .load(event.eventImageUrl)
                .into(EditEventImage)
    }

    override fun onButtonClick() {
        val editEventName = EditEventName?.text.toString()
        val editEventLocation = EditEventLocation?.text.toString()
        val editEventCategory = EditEventCategory?.text.toString()
        val editEventDescription = EditEventDescription?.text.toString()
        val editEventTime = EditEventTime?.text.toString()
        presenter.editEvent(event!!.id!!, editEventName, editEventDate!!, editEventTime, editEventLocation, editEventCategory, editEventDescription, imageUri)
    }

    override fun onBackPressed() {
        super.onBackPressed()
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
        overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                val returnIntent = Intent()
                setResult(Activity.RESULT_CANCELED, returnIntent)
                finish()
                overridePendingTransition(R.anim.slide_pop_enter_modal, R.anim.slide_pop_exit_modal)
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun setUp() {
        setSupportActionBar(EditEventToolbar)
        supportActionBar?.setHomeAsUpIndicator(R.drawable.navbar_ico_close)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setDisplayShowHomeEnabled(true)
    }

    private fun updateLabel() {
        val myFormat = "dd/MM/yy" //In which you need put here
        val sdf = SimpleDateFormat(myFormat, Locale.FRENCH)
        EditEventDate?.setText(sdf.format(myCalendar.time))
        editEventDate = myCalendar.time.time
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }

    override fun onEventEdited(eventMap: MutableMap<String, Any>) {
        val builder = AlertDialog.Builder(this)
        builder.setCancelable(false)
        builder.setMessage("Your event has been modified")
                .setPositiveButton("Accept", { _, _ ->
                    val returnIntent = Intent()
                    returnIntent.putExtra("result", eventMap as Serializable)
                    setResult(Activity.RESULT_OK, returnIntent)
                    finish()
                })
        // Create the AlertDialog object and return it
        builder.create()
        builder.show()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@EditEventActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Editing Your Event")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun showServiceError(message: String) {
        Snackbar.make(this.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
    }

    override fun showInvalidFieldsError() {
        Snackbar.make(this.findViewById(android.R.id.content), "Fields must not be empty", Snackbar.LENGTH_SHORT).show()
    }

    private fun BringImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 1000)
    }

    private fun modifyListeners() {
        EditEventName?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentNameModified = !currentName.equals(s.toString())
                checkEnabled()
            }
        })

        EditEventDate?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentDateModified = !currentDate.equals(s.toString())
                checkEnabled()
            }
        })

        EditEventTime.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentTimeModified = !currentTime.equals(s.toString())
                checkEnabled()
            }
        })

        EditEventLocation?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentLocationModified = !currentLocation.equals(s.toString())
                checkEnabled()
            }
        })

        EditEventCategory?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentCategoryModified = !currentCategory.equals(s.toString())
                checkEnabled()
            }
        })

        EditEventDescription?.addTextChangedListener(object : TextWatcher {

            override fun afterTextChanged(s: Editable) {}

            override fun beforeTextChanged(s: CharSequence, start: Int,
                                           count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int,
                                       before: Int, count: Int) {
                currentDescriptionModified = !currentDescription.equals(s.toString())
                checkEnabled()
            }
        })
    }

    fun checkEnabled() {
        if (currentNameModified || currentTimeModified || currentDateModified || currentCategoryModified || currentLocationModified || currentDescriptionModified || currentImageModified) {
            mSaveBtnEE?.setButtonEnabled(true)
        } else {
            mSaveBtnEE?.setButtonEnabled(false)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageUri = data.data//Getting uri of the data
                val imageStream = contentResolver.openInputStream(imageUri)//creating an imputstream
                val profilePicture = BitmapFactory.decodeStream(imageStream)
                EditEventImage?.setImageBitmap(profilePicture)
                IMAGE_STATUS = true//setting the flag
                currentImageModified = true
                checkEnabled()
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }

        }
    }

    companion object {
        const val TAG = "EditEventDetail"
        const val KEY_EVENT = "key_event"
    }

}
