package com.dev.lavin.bcard.main.events.edit

import android.net.Uri
import android.text.TextUtils
import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.domain2.interactors.events.EventsInteractor
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class EditEventPresenterImpl(editEventView: EditEventView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<EditEventView>(editEventView), EditEventPresenter, EventsInteractor.EventsEditInteractorResponse {

    @Inject
    lateinit var eventsInteractor: EventsInteractor

    init {
        view = editEventView
        basePresenterComponent.injectPresenter(this)
    }

    override fun editEvent(id: String, name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri?) {
        if (!TextUtils.isEmpty(name) && date != 0L && !TextUtils.isEmpty(time) && !TextUtils.isEmpty(location) && !TextUtils.isEmpty(category) && !TextUtils.isEmpty(description)) {
            view?.hideKeyboard()
            view?.showLoading()
            eventsInteractor.editEvent(id, name, date, time, location, category, description, url, this)
        } else {
            view?.showInvalidFieldsError()
        }
    }

    override fun onEventsEditInteractorResponseSuccess(eventMap: MutableMap<String, Any>) {
        view?.hideLoading()
        view.onEventEdited(eventMap)
    }

    override fun onEventsEditInteractorResponseFailure(message: String) {
        view?.hideLoading()
        view?.showServiceError(message)
    }

}