package com.dev.lavin.bcard.main.profile

import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.content.Intent
import android.net.Uri
import android.support.design.widget.BottomSheetDialog
import android.support.v7.widget.LinearLayoutManager
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.Toast
import com.bumptech.glide.Glide
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.adapters.CardsAdapter
import com.dev.lavin.bcard.base.BaseFragment
import com.dev.lavin.bcard.blueetooth.BluetoothShareActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.fragment.BaseFragmentModule
import com.dev.lavin.bcard.main.MainActivity
import com.dev.lavin.bcard.main.cards.create.CardCreateActivity
import com.dev.lavin.bcard.main.cards.detail.CardDetailActivity
import com.dev.lavin.domain2.model.Card
import com.google.firebase.auth.FirebaseAuth
import com.google.zxing.BarcodeFormat
import com.google.zxing.MultiFormatWriter
import com.google.zxing.WriterException
import com.google.zxing.common.BitMatrix
import com.journeyapps.barcodescanner.BarcodeEncoder
import kotlinx.android.synthetic.main.fragment_profile.*
import kotlinx.android.synthetic.main.fragment_profile_info.*
import org.json.JSONObject
import javax.inject.Inject


class ProfileFragment : BaseFragment(), ProfileView, CardsAdapter.OnCardSelectedListener {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var presenter: ProfilePresenter

    private var cardsProfileAdapter: CardsAdapter? = null

    private var mBluetoothAdapter: BluetoothAdapter? = null

    private var cardSelected: Card? = null

    override val xmlToInflate: Int
        get() = R.layout.fragment_profile


    override fun onBackPressed() {

    }

    override fun initDIComponent() {
        fragmentComponent = getParentComponent(ActivityComponent::class.java).with(BaseFragmentModule(this, activity!!))
        fragmentComponent!!.inject(this)
    }

    override fun initViews() {
        setHasOptionsMenu(true)
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
        if (mAuth.currentUser != null) {
            presenter.getUserInfo(mAuth.currentUser!!.uid)
            presenter.getUserCards(mAuth.currentUser!!.uid)
            sendTocreateCard()
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?, inflater: MenuInflater?) {
        inflater!!.inflate(R.menu.profile_menu, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        return when (item!!.itemId) {
            R.id.action_log_out_btn -> {
                logOut()
                true
            }
            else -> false
        }
    }

    override fun logOut() {
        (activity as MainActivity).logOut()
    }

    override fun setEmptyCards() {
        ProfileEmpty.visibility = View.VISIBLE
        CardListProfile.visibility = View.GONE
    }

    override fun showError(message: String) {
        /*if (view != null) {
            Snackbar.make(view!!.findViewById(android.R.id.content), message, Snackbar.LENGTH_SHORT).show()
        }*/
        if (context != null) {
            Toast.makeText(context, message, Toast.LENGTH_LONG).show()
        }
    }

    override fun setProfileInfo(name: String?, url: String?) {
        //ProfileName?.text = name
        if (url != null && context != null) {
            val uri = Uri.parse(url)
            Glide.with(context!!).load(uri).into(ProfilePhoto)
        }
        if (name != null) {
            ProfileName.text = name
        }
    }

    override fun setCardList(cardList: MutableList<Card>) {
        ProfileEmpty?.visibility = View.GONE
        CardListProfile?.visibility = View.VISIBLE
        cardsProfileAdapter = CardsAdapter(cardList!!, this)
        CardListProfile?.layoutManager = LinearLayoutManager(context)
        CardListProfile?.adapter = cardsProfileAdapter
        CardListProfile?.setHasFixedSize(true)
        cardsProfileAdapter?.notifyDataSetChanged()
    }

    fun sendToDetail(card: Card) {
        val intent = Intent(context, CardDetailActivity::class.java)
        intent.putExtra(CardDetailActivity.KEY_CARD, card)
        intent.putExtra(CardDetailActivity.KEY_CARD_FROM, false)
        startActivity(intent)
        activity?.overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    fun sendTocreateCard() {
        ProfileAddCard.setOnClickListener { sendToCreate() }
    }

    fun sendToCreate() {
        val intent = Intent(context, CardCreateActivity::class.java)
        startActivity(intent)
        activity?.overridePendingTransition(R.anim.slide_enter_modal, R.anim.slide_exit_modal)
    }

    override fun onShareCardSelected(card: Card) {
        cardSelected = card
        val contentView = layoutInflater.inflate(R.layout.fragment_bottom_sheet_dialog, null)
        val dialog = BottomSheetDialog(context!!)
        dialog.setContentView(contentView)
        dialog.show()
        val qr = contentView.findViewById(R.id.bottom_sheet_qr) as LinearLayout
        val bluetooth = contentView.findViewById(R.id.bottom_sheet_bluetooth) as LinearLayout
        qr.setOnClickListener {
            dialog.dismiss()
            val contentViewQR = layoutInflater.inflate(R.layout.fragment_bottom_sheet_qr_dialog, null)
            val dialogQR = BottomSheetDialog(context!!)
            dialogQR.setContentView(contentViewQR)
            dialogQR.show()
            val qr = contentViewQR.findViewById(R.id.qr_container) as ImageView
            val multiFormatWriter = MultiFormatWriter()
            try {
                val json = JSONObject()
                json.put("bwcardid", card!!.id)
                json.put("bwcardusid", card!!.userId)
                val bitMatrix: BitMatrix = multiFormatWriter.encode(json.toString(), BarcodeFormat.QR_CODE, 200, 200)
                val barEncoder = BarcodeEncoder()
                val bitMap = barEncoder.createBitmap(bitMatrix)
                qr.setImageBitmap(bitMap)
            } catch (e: WriterException) {
                e.printStackTrace()
            }
        }
        bluetooth.setOnClickListener {
            // Get local Bluetooth adapter
            if (mBluetoothAdapter == null || !(mBluetoothAdapter!!.isEnabled)) {
                val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
                startActivityForResult(enableBtIntent, ProfileFragment.REQUEST_ENABLE_BT)
                dialog.dismiss()
            } else {
                val intent = Intent(activity, BluetoothShareActivity::class.java)
                intent.putExtra(BluetoothShareActivity.KEY_CARD, card)
                startActivity(intent)
                dialog.dismiss()
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == ProfileFragment.REQUEST_ENABLE_BT) {
            // When the request to enable Bluetooth returns
            if (resultCode == Activity.RESULT_OK) {
                // Bluetooth is now enabled, so set up a chat session
                val intent = Intent(activity, BluetoothShareActivity::class.java)
                intent.putExtra(BluetoothShareActivity.KEY_CARD, cardSelected)
                startActivity(intent)
            } else {
                // User did not enable Bluetooth or an error occurred
                Toast.makeText(activity, "You must enable bluetooth to share and receive cards",
                        Toast.LENGTH_SHORT).show()
            }
        }
    }//onActivityResult

    override fun onCardSelected(card: Card) {
        sendToDetail(card)
    }

    companion object {
        const val REQUEST_ENABLE_BT: Int = 12
        fun newInstance(): ProfileFragment {
            return ProfileFragment()
        }
    }
}
