package com.dev.lavin.bcard.main.profile

import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.bcard.utils.CardUtils
import com.dev.lavin.domain2.interactors.cards.CardsInteractor
import com.dev.lavin.domain2.interactors.profile.ProfileInteractor
import com.dev.lavin.domain2.model.Card
import com.dev.lavin.domain2.model.User
import java.util.*
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class ProfilePresenterImpl(profileView: ProfileView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<ProfileView>(profileView), ProfilePresenter, ProfileInteractor.ProfileInteractorResponse, CardsInteractor.CardsFromUserInteractorResponse {

    @Inject
    lateinit var profileInteractor: ProfileInteractor

    @Inject
    lateinit var cardsInteractor: CardsInteractor

    private var cardListProfileFragment: MutableList<Card>? = ArrayList()

    init {
        view = profileView
        basePresenterComponent.injectPresenter(this)
    }

    override fun getUserInfo(id: String) {
        profileInteractor.getProfileInfo(id, this)
    }

    override fun onProfileSuccesResponse(user: User) {
        view?.setProfileInfo(user.name, user.image)
    }

    override fun onCardsFromUserEmpty() {
        view?.setEmptyCards()
    }

    override fun onProfileUserNotFound() {
        view?.logOut()
    }

    override fun onProfileFailureAuth(message: String) {
        view?.showError(message)
    }

    override fun getUserCards(id: String) {
        cardsInteractor.getUserCards(id, this)
    }

    override fun onCardsFromUserInteractorResponseSuccess(cardList: MutableMap<String, MutableList<Card>>) {
        CardUtils.addCards(cardList, cardListProfileFragment)
        CardUtils.modifyCards(cardList, cardListProfileFragment)
        view?.setCardList(cardListProfileFragment!!)
    }

    override fun onCardsFromUserInteractorResponseFailure(message: String) {
        view?.showError(message)
    }
}