package com.dev.lavin.bcard.register

import android.app.Activity
import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Bundle
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.login.LoginActivity
import com.dev.lavin.bcard.main.MainActivity
import com.google.firebase.auth.FirebaseAuth
import kotlinx.android.synthetic.main.activity_register.*
import java.io.FileNotFoundException
import javax.inject.Inject

class RegisterActivity : BaseActivity<ActivityComponent>(), RegisterView {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var presenter: RegisterPresenter

    private var imageUri: Uri? = null

    private var IMAGE_STATUS = false

    private var progressDialog: ProgressDialog? = null

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_register)
        setUp()
    }

    override fun setUp() {
        setSupportActionBar(mToolbarRg)
        supportActionBar?.title = ""
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setDisplayShowHomeEnabled(true)
        setRegisterAction()
        setPickerAction()
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@RegisterActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Creating your user")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun showServiceError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_LONG).show()
    }

    override fun showInvalidFieldsError() {
        Toast.makeText(this, "Fields must not be empty", Toast.LENGTH_LONG).show()
    }

    override fun showPassworError() {
        Toast.makeText(this, "Passwords are not equal", Toast.LENGTH_LONG).show()
    }

    override fun showGenericError() {
        Toast.makeText(this, "An unexpected error occurred", Toast.LENGTH_LONG).show()
    }

    override fun showPassworLengthError() {
        Toast.makeText(this, "Passwords length must be greater than 7", Toast.LENGTH_LONG).show()
    }

    override fun showInvalidEmailError() {
        Toast.makeText(this, "Enter a valid email", Toast.LENGTH_LONG).show()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                sendToLogin()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        sendToLogin()
    }

    override fun onStart() {
        super.onStart()
        val user = mAuth.currentUser
        if (user != null) {
            //  No user is signed in
            sendToMain()
        }
    }

    private fun setRegisterAction() {
        mRegisterBtnRg?.setOnClickListener {
            val registerEmail = mEmailEdtRg?.text.toString()
            val registerPass = mPassEdtRg?.text.toString()
            val registerConfirmPass = mConfirmPassEdtRg?.text.toString()
            val registerName = mNameEdtRg?.text.toString()
            presenter.tryRegister(registerEmail, registerPass, registerConfirmPass, registerName, imageUri)
        }
    }

    private fun setPickerAction(){
        mRegisterSetupImageRg.setOnClickListener { BringImagePicker()  }
    }

    private fun sendToLogin() {
        val intent = Intent(this, LoginActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_from_left, R.anim.slide_to_right)
        finish()
    }

    override fun sendToMain() {
        val intent = Intent(this, MainActivity::class.java)
        startActivity(intent)
        finish()
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }

    private fun BringImagePicker() {
        val intent = Intent(Intent.ACTION_GET_CONTENT)
        intent.type = "image/*"
        startActivityForResult(intent, 1000)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1000 && resultCode == Activity.RESULT_OK && data != null) {
            //Image Successfully Selected
            try {
                //parsing the Intent data and displaying it in the imageView
                imageUri = data.data//Getting uri of the data
                val imageStream = contentResolver.openInputStream(imageUri)//creating an imputstream
                val profilePicture = BitmapFactory.decodeStream(imageStream)
                mRegisterSetupImageRg?.setImageBitmap(profilePicture)
                IMAGE_STATUS = true//setting the flag
            } catch (e: FileNotFoundException) {
                e.printStackTrace()
            }

        }
    }
}
