package com.dev.lavin.bcard.register

import android.net.Uri
import android.text.TextUtils
import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.domain2.interactors.register.RegisterInteractor
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class RegisterPresenterImpl(registerView: RegisterView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<RegisterView>(registerView), RegisterPresenter, RegisterInteractor.RegisterInteractorResponse {

    @Inject
    lateinit var registerInteractor: RegisterInteractor

    init {
        view = registerView
        basePresenterComponent.injectPresenter(this)
    }

    override fun tryRegister(user: String, pass: String, confirmPass: String, name: String, url: Uri?) {
        if (!TextUtils.isEmpty(user) && !TextUtils.isEmpty(pass) && !TextUtils.isEmpty(confirmPass) && !TextUtils.isEmpty(name) && url != null) {
            if (isEmailValid(user)) {
                if (isPasswordValid(pass)) {
                    if (pass == confirmPass) {
                        view?.hideKeyboard()
                        view?.showLoading()
                        registerInteractor.register(user, pass, name, url, this)
                    } else {
                        view?.showPassworError()
                    }
                } else {
                    view?.showPassworLengthError()
                }
            } else {
                view?.showInvalidEmailError()
            }
        } else {
            view?.showInvalidFieldsError()
        }
    }

    override fun onRegisterInteractorSuccesResponse() {
        view?.hideLoading()
        view?.sendToMain()
    }

    override fun onRegisterInteractorFailureAuth(message: String) {
        view?.hideLoading()
        if (message != null) {
            view?.showServiceError(message)
        } else {
            view?.showGenericError()
        }
    }

    override fun onRegisterInteractorImageFailure(message: String) {
        view?.hideLoading()
        if (message != null) {
            view?.showServiceError(message)
        } else {
            view?.showGenericError()
        }
    }

    private fun isEmailValid(email: String): Boolean {
        return email.contains("@")
    }

    private fun isPasswordValid(password: String): Boolean {
        return password.length > 7
    }

}