package com.dev.lavin.bcard.resetpassword

import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.Snackbar
import android.view.MenuItem
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import com.dev.lavin.bcard.R
import com.dev.lavin.bcard.base.BaseActivity
import com.dev.lavin.bcard.di.activity.ActivityComponent
import com.dev.lavin.bcard.di.activity.ActivityModule
import com.dev.lavin.bcard.login.LoginActivity
import com.dev.lavin.bcard.main.MainActivity
import com.dev.lavin.bcard.register.RegisterActivity
import com.google.firebase.auth.FirebaseAuth
import kotlinx.android.synthetic.main.activity_reset_password.*
import javax.inject.Inject


class ResetPasswordActivity : BaseActivity<ActivityComponent>(), ResetPasswordView {

    @Inject
    lateinit var mAuth: FirebaseAuth

    private var progressDialog: ProgressDialog? = null

    @Inject
    lateinit var presenter: ResetPasswordPresenter

    override fun initDI() {
        activityComponent = appComponent?.with(ActivityModule(this, this))
        activityComponent?.injectActivity(this)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_reset_password)
        setResetPasswordAction()
        setUp()
    }

    override fun onBackPressed() {
        super.onBackPressed()
        sendToLogin()
    }

    override fun onStart() {
        super.onStart()
        val user = mAuth.currentUser
        if (user != null) {
            //  No user is signed in
            sendToMain()
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                sendToLogin()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    override fun setUp() {
        setSupportActionBar(mToolbarRp)
        supportActionBar?.title = ""
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        supportActionBar?.setDisplayShowHomeEnabled(true)
    }

    override fun showLoading() {
        progressDialog = ProgressDialog(this@ResetPasswordActivity)
        progressDialog?.setTitle("Please Wait")
        progressDialog?.setCancelable(false)
        progressDialog?.setMessage("Processing your request")
        progressDialog?.show()
    }

    override fun hideLoading() {
        progressDialog?.dismiss()
    }

    override fun showUserNotFound() {
        Snackbar.make(findViewById(android.R.id.content), "Account Not Found", Snackbar.LENGTH_LONG).setAction("Create One") {
            sendToRegister()
        }.show()
    }

    override fun showError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_LONG).show()
    }

    override fun onResetPassWordSucces() {
        mEmailEdtRp?.text?.clear()
        Snackbar.make(findViewById(android.R.id.content), "Password Reset Mail Send Successfully", Snackbar.LENGTH_LONG).setAction("Log In") {
            sendToLogin()
        }.show()
    }

    override fun showInvalidEmail() {
        Toast.makeText(this, "Enter a valid email", Toast.LENGTH_LONG).show()
    }

    private fun setResetPasswordAction() {
        mResetPasswordBtnRp.setOnClickListener {
            //onClickListener for password reset email send button
            presenter.tryResetPassword(mEmailEdtRp.text.toString())
        }
    }

    private fun sendToLogin() {
        val intent = Intent(this, LoginActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_from_left, R.anim.slide_to_right)
        finish()
    }

    private fun sendToRegister() {
        val intent = Intent(this, RegisterActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_up, R.anim.slide_out_up)
        finish()
    }

    private fun sendToMain() {
        val intent = Intent(this, MainActivity::class.java)
        startActivity(intent)
        finish()
    }

    override fun hideKeyboard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager?
        imm?.hideSoftInputFromWindow(currentFocus.windowToken, 0)
    }
}
