package com.dev.lavin.bcard.resetpassword

import com.dev.lavin.bcard.base.BasePresenterImpl
import com.dev.lavin.bcard.di.presenter.BasePresenterComponent
import com.dev.lavin.bcard.utils.ValidationUtils
import com.dev.lavin.domain2.interactors.resetpassword.ResetPasswordInteractor
import javax.inject.Inject

/**
 * Created by lavin
 * on 16/04/2018.
 */
class ResetPasswordPresenterImpl(resetPasswordView: ResetPasswordView, basePresenterComponent: BasePresenterComponent) : BasePresenterImpl<ResetPasswordView>(resetPasswordView), ResetPasswordPresenter, ResetPasswordInteractor.ResetPasswordInteractorResponse {

    @Inject
    lateinit var resetPasswordInteractor: ResetPasswordInteractor

    init {
        view = resetPasswordView
        basePresenterComponent.injectPresenter(this)
    }

    override fun tryResetPassword(email: String) {
        if (ValidationUtils.validateEmail(email)) {
            view?.hideKeyboard()
            view?.showLoading()
            resetPasswordInteractor.resetPassword(email, this)
        } else {
            view?.showInvalidEmail()
        }
    }

    override fun onResetPasswordSuccesResponse() {
        view?.hideLoading()
        view?.onResetPassWordSucces()
    }

    override fun onResetPasswordFailure(message: String) {
        view?.hideLoading()
        view?.showError(message)
    }

    override fun onResetPasswordUserNotFound() {
        view?.hideLoading()
        view?.showUserNotFound()
    }
}