package com.dev.lavin.bcard.utils

import android.app.Activity
import android.content.Context
import android.support.design.widget.AppBarLayout
import android.support.v4.content.ContextCompat
import android.support.v4.widget.NestedScrollView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.animation.DecelerateInterpolator
import android.view.animation.TranslateAnimation
import android.widget.FrameLayout
import com.dev.lavin.bcard.R
import kotlinx.android.synthetic.main.custom_floating_button.view.*
import java.lang.ref.WeakReference


class FloatingButton : FrameLayout {
    // ---- Attributes ----

    private val TIME_TO_SHOW: Long = 300
    private val TIME_STEP: Long = 20

    private var mEvents: IEvents? = null
    private var mTimeFromHide: Long = 0
    private val mVerticalOffset = 0
    private var mIsButtonsShown = true
    private var mIsButtonHideEnable: Boolean = false
    private var mHideWatcher: HideWatcher? = null

    private var mButtonText: String? = null
    private var mButtonBackgroundColor: Int = 0
    private var mButtonTextColor: Int = 0

    // ---- END Attrbutes ----

    // ---- Builder ----

    constructor(context: Context) : super(context) {
        initLayout()
    }

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {
        initLayout()
        initAttr(attrs)
    }

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        initLayout()
        initAttr(attrs)
    }

    // ---- END Builder ----

    // ---- FrameLayout ----

    override fun onFinishInflate() {
        super.onFinishInflate()
        initComponents()
    }

    // ---- END FrameLayout ----

    // ---- Public ----


    fun setButtonHideEnable(hideEnable: Boolean) {
        mIsButtonHideEnable = hideEnable
        if (hideEnable) {
            setBackgroundResource(R.drawable.floating_button_layer)
        } else {
            background = null
        }
    }

    private fun hideButtonIfNeeded() {
        if (!mIsButtonHideEnable) {
            return
        }
        if (mHideWatcher == null) {
            mHideWatcher = HideWatcher(this)
        }
        mTimeFromHide = 0
        if (!mHideWatcher!!.isAlive) {
            mHideWatcher = HideWatcher(this)
            mHideWatcher!!.start()
        }
        hideButton()
    }

    fun setupScrollListeners(appBarLayout: AppBarLayout?, nested: NestedScrollView?) {
        appBarLayout?.addOnOffsetChangedListener(AppBarLayout.OnOffsetChangedListener { _, verticalOffset ->
            if (mVerticalOffset == verticalOffset) {
                return@OnOffsetChangedListener
            }
            hideButtonIfNeeded()
        })

        nested?.setOnScrollChangeListener(NestedScrollView.OnScrollChangeListener { _, _, _, _, _ -> hideButtonIfNeeded() })
    }

    fun isButtonEnabled(): Boolean {
        return if (mfb_button != null) {
            mfb_button.isEnabled
        } else isEnabled
    }

    fun setButtonEnabled(enabled: Boolean) {
        if (mfb_button != null) {
            mfb_button.isEnabled = enabled
        }
    }

    fun setEvents(events: IEvents) {
        mEvents = events
    }

    fun setButtonClick() {
        if (mEvents != null) {
            mEvents!!.onButtonClick()
        }
    }

    // ---- END Public ----

    // ---- Private ----

    private fun initComponents() {
        if (mButtonText != null) {
            mfb_button?.text = mButtonText
        }
        mfb_button?.background = ContextCompat.getDrawable(context, mButtonBackgroundColor)
        mfb_button?.setTextColor(ContextCompat.getColor(context, mButtonTextColor))
        mfb_button?.setOnClickListener {
            if (mEvents != null) {
                mEvents!!.onButtonClick()
            }
        }
    }

    private fun initAttr(attrs: AttributeSet?) {
        if (attrs != null) {
            val a = context.theme.obtainStyledAttributes(
                    attrs,
                    R.styleable.floating_button,
                    0, 0)
            try {
                mButtonText = a.getString(R.styleable.floating_button_floating_button_text)
                mButtonBackgroundColor = a.getResourceId(R.styleable.floating_button_floating_button_background, R.drawable.buttonpinkrounded)
                mButtonTextColor = a.getResourceId(R.styleable.floating_button_floating_button_text_color, R.color.white)

            } finally {
                a.recycle()
            }
        }
    }

    private fun initLayout() {
        val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        inflater.inflate(R.layout.custom_floating_button, this)
    }

    private fun showButton() {
        if (mIsButtonsShown) {
            return
        }
        mIsButtonsShown = true
        val goUp = TranslateAnimation(
                TranslateAnimation.RELATIVE_TO_SELF, 0f, TranslateAnimation.RELATIVE_TO_SELF, 0f,
                TranslateAnimation.RELATIVE_TO_SELF, 1f, TranslateAnimation.RELATIVE_TO_SELF, 0f)
        goUp.fillAfter = true
        goUp.fillBefore = true
        goUp.duration = 300
        goUp.interpolator = DecelerateInterpolator()

        var activity: Activity? = null
        if (context != null && context is Activity) {
            activity = context as Activity
        }
        if (activity != null) {
            activity.runOnUiThread { startAnimation(goUp) }
        }
    }

    private fun hideButton() {
        if (!mIsButtonsShown) {
            return
        }
        mIsButtonsShown = false
        val goDown = TranslateAnimation(
                TranslateAnimation.RELATIVE_TO_SELF, 0f, TranslateAnimation.RELATIVE_TO_SELF, 0f,
                TranslateAnimation.RELATIVE_TO_SELF, 0f, TranslateAnimation.RELATIVE_TO_SELF, 1f)
        goDown.fillAfter = true
        goDown.fillBefore = true
        goDown.duration = 300
        goDown.interpolator = DecelerateInterpolator()

        var activity: Activity? = null
        if (context != null && context is Activity) {
            activity = context as Activity
        }
        if (activity != null) {
            activity.runOnUiThread { startAnimation(goDown) }
        }
    }

    // ---- END Private ----

    // ---- Interface ----

    interface IEvents {
        fun onButtonClick()
    }

    // ---- END Interface ----

    // ---- Class ----

    internal inner class HideWatcher(mfbButton: FloatingButton) : Thread() {
        private var mReference: WeakReference<FloatingButton> = WeakReference(mfbButton)

        override fun run() {
            while (mReference.get() != null
                    && mReference.get()!!.mTimeFromHide < TIME_TO_SHOW
                    && mReference.get()!!.mIsButtonHideEnable) {
                mTimeFromHide += TIME_STEP
                try {
                    sleep(TIME_STEP)
                } catch (e: InterruptedException) {
                    e.printStackTrace()
                }

            }
            if (mReference.get() != null) {
                showButton()
            }

        }
    }

    // ---- END Class ----
}