package com.dev.lavin.bcard.utils

import android.app.Activity
import android.content.Context
import android.graphics.Point
import android.os.Build
import android.support.v4.view.MarginLayoutParamsCompat
import android.support.v4.view.ViewCompat
import android.util.DisplayMetrics
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.RelativeLayout

/**
 * Created by koke07
 * on 15/11/16.
 */

object ScreenUtils {

    private var screenWidth = 0

    fun dp2pixels(dp: Float, context: Context): Int {
        return TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dp, context.resources.displayMetrics).toInt()
    }

    /**
     * Returns scale factor to scale dimensions uniformly (maintain the aspect ratio) so
     * that both dimensions (width and height) will be equal to or larger than given
     * <param></param>dstWidth and <param></param>dstHeight.
     * @param width is the width of the screen
     * @param height is the height of the screen
     * @param dstWidth is the dstWidth of the screen
     * @param dstHeight is the dstHeigth of the screen
     * @return a float that indicates the scale factor
     */
    fun getScaleCropFactor(width: Int, height: Int, dstWidth: Int, dstHeight: Int): Float {
        var scaleFactor = 1f

        // Let's check some things and calc scale.
        if (width.toFloat() != 0f && height.toFloat() != 0f && dstWidth.toFloat() != 0f && dstHeight.toFloat() != 0f) {
            scaleFactor = Math.max(dstWidth.toFloat() / width.toFloat(), dstHeight.toFloat() / height.toFloat())
        }
        return scaleFactor
    }

    fun getMeasuredWidth(v: View?): Int {
        return v?.measuredWidth ?: 0
    }

    fun getWidth(v: View?): Int {
        return v?.width ?: 0
    }

    fun getWidthWithMargin(v: View): Int {
        return getWidth(v) + getMarginHorizontally(v)
    }

    @JvmOverloads
    fun getStart(v: View?, withoutPadding: Boolean = false): Int {
        if (v == null) {
            return 0
        }
        return if (isLayoutRtl(v)) {
            if (withoutPadding) v.right - getPaddingStart(v) else v.right
        } else {
            if (withoutPadding) v.left + getPaddingStart(v) else v.left
        }
    }

    @JvmOverloads
    fun getEnd(v: View?, withoutPadding: Boolean = false): Int {
        if (v == null) {
            return 0
        }
        return if (isLayoutRtl(v)) {
            if (withoutPadding) v.left + getPaddingEnd(v) else v.left
        } else {
            if (withoutPadding) v.right - getPaddingEnd(v) else v.right
        }
    }

    fun getPaddingStart(v: View?): Int {
        return if (v == null) {
            0
        } else ViewCompat.getPaddingStart(v)
    }

    fun getPaddingEnd(v: View?): Int {
        return if (v == null) {
            0
        } else ViewCompat.getPaddingEnd(v)
    }

    fun getPaddingHorizontally(v: View?): Int {
        return if (v == null) {
            0
        } else v.paddingLeft + v.paddingRight
    }

    fun getMarginStart(v: View?): Int {
        if (v == null) {
            return 0
        }
        val lp = v.layoutParams as ViewGroup.MarginLayoutParams
        return MarginLayoutParamsCompat.getMarginStart(lp)
    }

    fun getMarginEnd(v: View?): Int {
        if (v == null) {
            return 0
        }
        val lp = v.layoutParams as ViewGroup.MarginLayoutParams
        return MarginLayoutParamsCompat.getMarginEnd(lp)
    }

    fun getMarginHorizontally(v: View?): Int {
        if (v == null) {
            return 0
        }
        val lp = v.layoutParams as ViewGroup.MarginLayoutParams
        return MarginLayoutParamsCompat.getMarginStart(lp) + MarginLayoutParamsCompat.getMarginEnd(lp)
    }

    fun isLayoutRtl(v: View): Boolean {
        return ViewCompat.getLayoutDirection(v) == ViewCompat.LAYOUT_DIRECTION_RTL
    }

    fun getScreenWidth(c: Context): Int {
        if (screenWidth == 0) {
            val wm = c.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            val display = wm?.defaultDisplay
            if (display != null) {
                val size = Point()
                display.getSize(size)
                screenWidth = size.x
            }
        }

        return screenWidth
    }

    fun getSoftbuttonsbarHeight(activity: Activity): Int {
        val metrics = DisplayMetrics()
        activity.windowManager.defaultDisplay.getMetrics(metrics)
        val usableHeight = metrics.heightPixels
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            activity.windowManager.defaultDisplay.getRealMetrics(metrics)
        }
        val realHeight = metrics.heightPixels
        return if (realHeight > usableHeight) {
            realHeight - usableHeight
        } else {
            0
        }
    }

    fun softButtonsBarManager(activity: Activity, context: Context, v: View) {
        val display = activity.windowManager.defaultDisplay
        val size = Point()
        display.getSize(size)

        val factor = (size.y - ScreenUtils.getSoftbuttonsbarHeight(activity)).toFloat() / size.y.toFloat()
        v.scaleX = factor
        v.scaleY = factor

        val marginParams = ViewGroup.MarginLayoutParams(v.layoutParams)
        val logoLayout = v.layoutParams as RelativeLayout.LayoutParams

        marginParams.setMargins(logoLayout.leftMargin, (ScreenUtils.dp2pixels(38f, context) * factor).toInt(), logoLayout.rightMargin, logoLayout.bottomMargin)
        val layoutParams = RelativeLayout.LayoutParams(marginParams)
        layoutParams.addRule(RelativeLayout.CENTER_HORIZONTAL)
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_TOP)
        v.layoutParams = layoutParams
    }

    fun lowDensityDisplay(activity: Activity, context: Context, v: View) {
        val display = activity.windowManager.defaultDisplay
        val size = Point()
        display.getSize(size)

        v.scaleX = 0.9f
        v.scaleY = 0.84f

        val marginParams = ViewGroup.MarginLayoutParams(v.layoutParams)
        val logoLayout = v.layoutParams as RelativeLayout.LayoutParams

        marginParams.setMargins(logoLayout.leftMargin, ScreenUtils.dp2pixels(27f, context), logoLayout.rightMargin, logoLayout.bottomMargin)
        val layoutParams = RelativeLayout.LayoutParams(marginParams)
        layoutParams.addRule(RelativeLayout.CENTER_HORIZONTAL)
        layoutParams.addRule(RelativeLayout.ALIGN_PARENT_TOP)
        v.layoutParams = layoutParams
    }
}
