package com.dev.lavin.data.services


import android.app.Activity
import android.net.Uri
import android.util.Log
import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.data.entities.card.CardEntity
import com.dev.lavin.data.entities.event.EventEntity
import com.dev.lavin.data.entities.mappers.CardToCardEntityMapper
import com.dev.lavin.domain2.boundaries.CardsRepoInterface
import com.dev.lavin.domain2.model.Card
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.DocumentChange
import com.google.firebase.firestore.FieldValue
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.storage.FirebaseStorage
import java.sql.Timestamp
import javax.inject.Inject


/**
 * Created by Jorge Lavin on 16/04/18.
 */

class CardsService(baseRepositoryComponent: BaseRepositoryComponent, private var activty: Activity) : CardsRepoInterface {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var mFireStore: FirebaseFirestore

    @Inject
    lateinit var mFireStorage: FirebaseStorage

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun getCards(cardsResponseListener: CardsRepoInterface.CardsResponseListener) {
        val query = mFireStore.collection("Cards")
        query.addSnapshotListener { documentSnapshots, e ->
            // Handle errors
            if (e != null) {
                cardsResponseListener.onCardsFailure(e.message!!)
            } else {
                if (!documentSnapshots!!.isEmpty) {
                    val cardList: MutableMap<String, MutableList<Card>> = mutableMapOf()
                    val cardListAdded: MutableList<Card> = ArrayList()
                    val cardListModified: MutableList<Card> = ArrayList()
                    for (doc in documentSnapshots.documentChanges) {
                        if (doc.type == DocumentChange.Type.ADDED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardListAdded.add(mapper.reverseMap(cardEntity))
                        }
                        if (doc.type == DocumentChange.Type.MODIFIED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardListModified.add(mapper.reverseMap(cardEntity))
                        }
                    }
                    cardList["added"] = cardListAdded
                    cardList["modified"] = cardListModified
                    cardsResponseListener.onCardsSuccesResponse(cardList)
                }
            }
        }
    }

    override fun getContactCards(id: String, cardsContactResponseListener: CardsRepoInterface.CardsContactResponseListener) {
        val query = mFireStore.collection("Cards").whereEqualTo("users.$id", true)
        val cardContactsList: MutableMap<String, MutableList<Card>> = mutableMapOf()
        query.addSnapshotListener { documentSnapshots, e ->
            // Handle errors
            if (e != null) {
                cardsContactResponseListener.onCardsContactFailure(e.message!!)
            } else {
                if (!documentSnapshots!!.isEmpty) {
                    val cardContactsListAdded: MutableList<Card> = ArrayList()
                    val cardContactsListModified: MutableList<Card> = ArrayList()
                    for (doc in documentSnapshots.documentChanges) {
                        if (doc.type == DocumentChange.Type.ADDED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardContactsListAdded.add(mapper.reverseMap(cardEntity))
                        }
                        if (doc.type == DocumentChange.Type.MODIFIED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardContactsListModified.add(mapper.reverseMap(cardEntity))
                        }
                    }
                    cardContactsList["added"] = cardContactsListAdded
                    cardContactsList["modified"] = cardContactsListModified
                    cardsContactResponseListener.onCardsContactSuccesResponse(cardContactsList)
                } else {
                    cardsContactResponseListener.onCardsContactSuccesResponse(cardContactsList)
                }
            }
        }
    }

    override fun getUserCards(id: String, cardsFromUserResponseListener: CardsRepoInterface.CardsFromUserResponseListener) {
        val query = mFireStore.collection("Cards").whereEqualTo("user_id", id)
        query.addSnapshotListener { documentSnapshots, e ->
            // Handle errors
            if (e != null) {
                cardsFromUserResponseListener.onCardsFromUserFailure(e.message!!)
            } else {
                if (!documentSnapshots!!.isEmpty) {
                    val cardFromUserList: MutableMap<String, MutableList<Card>> = mutableMapOf()
                    val cardFromUserListAdded: MutableList<Card> = ArrayList()
                    val cardFromUserListModified: MutableList<Card> = ArrayList()
                    for (doc in documentSnapshots.documentChanges) {
                        if (doc.type == DocumentChange.Type.ADDED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardFromUserListAdded.add(mapper.reverseMap(cardEntity))
                        }
                        if (doc.type == DocumentChange.Type.MODIFIED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardFromUserListModified.add(mapper.reverseMap(cardEntity))
                        }
                    }
                    cardFromUserList["added"] = cardFromUserListAdded
                    cardFromUserList["modified"] = cardFromUserListModified
                    cardsFromUserResponseListener.onCardsFromUserSuccesResponse(cardFromUserList)
                } else {
                    cardsFromUserResponseListener.onCardsFromUserEmpty()
                }
            }
        }
    }

    override fun getCardsFromEvent(id: String, cardsFromEventrResponseListener: CardsRepoInterface.CardsFromEventrResponseListener) {
        val query = mFireStore.collection("Cards").whereEqualTo("events.$id", true)
        query.addSnapshotListener { documentSnapshots, e ->
            // Handle errors
            if (e != null) {
                cardsFromEventrResponseListener.onCardsFromEventFailure(e.message!!)
            } else {
                if (!documentSnapshots!!.isEmpty) {
                    val cardFromEventList: MutableMap<String, MutableList<Card>> = mutableMapOf()
                    val cardFromEventListAdded: MutableList<Card> = ArrayList()
                    val cardFromEventListModified: MutableList<Card> = ArrayList()
                    for (doc in documentSnapshots.documentChanges) {
                        if (doc.type == DocumentChange.Type.ADDED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardFromEventListAdded.add(mapper.reverseMap(cardEntity))
                        }
                        if (doc.type == DocumentChange.Type.MODIFIED) {
                            val cardId = doc.document.id
                            val cardEntity: CardEntity = doc.document.toObject(CardEntity::class.java).withId(cardId)
                            val mapper = CardToCardEntityMapper()
                            cardFromEventListModified.add(mapper.reverseMap(cardEntity))
                        }
                    }
                    cardFromEventList["added"] = cardFromEventListAdded
                    cardFromEventList["modified"] = cardFromEventListModified
                    cardsFromEventrResponseListener.onCardsFromEventSuccesResponse(cardFromEventList)
                } else {
                    cardsFromEventrResponseListener.onCardsFromEventEmpty()
                }
            }
        }
    }

    override fun setCardInEvent(idEvent: String, idCard: String, cardInEventResponseListener: CardsRepoInterface.CardInEventResponseListener) {
        val cardDocRef = mFireStore.collection("Cards").document(idCard)
        val eventDocRef = mFireStore.collection("Events").document(idEvent)
        mFireStore.runTransaction { transaction ->
            val eventEntity = transaction.get(eventDocRef).toObject(EventEntity::class.java)
            val cardEntity = transaction.get(cardDocRef).toObject(CardEntity::class.java)
            val cardsMap: HashMap<String, Boolean>
            val eventsMap: HashMap<String, Boolean>
            if (eventEntity?.cards != null) {
                eventEntity.cards!![idCard] = true
            } else {
                cardsMap = HashMap()
                cardsMap[idCard] = true
                eventEntity?.cards = cardsMap
            }
            if (cardEntity?.events != null) {
                cardEntity.events!![idEvent] = true
            } else {
                eventsMap = HashMap()
                eventsMap[idEvent] = true
                cardEntity?.events = eventsMap
            }
            transaction.update(cardDocRef, "events", cardEntity!!.events)
            transaction.update(eventDocRef, "cards", eventEntity!!.cards)
            null
        }.addOnSuccessListener {
            Log.d("updateCard", "Transaction success!")
            cardInEventResponseListener.onCardInEventSuccesResponse()
        }.addOnFailureListener { e ->
            Log.w("updateCard", "Transaction failure.", e)
            cardInEventResponseListener.onCardInEventFailure(e.message!!)
        }
    }

    override fun createCard(name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardCreatedResponseListener: CardsRepoInterface.CardCreatedResponseListener) {
        val myStorageReference = mFireStorage.getReference("card_images")
        val uploadReference = myStorageReference.child(name + Timestamp(System.currentTimeMillis()))
        val uploadCardProfileImage = uploadReference.putFile(profile!!)
        var downloadUrl: Uri?
        var downloadCaptionUrl: Uri? = null
        uploadCardProfileImage.addOnCompleteListener { uploadTask ->
            if (uploadTask.isSuccessful) {
                downloadUrl = uploadTask.result.downloadUrl
                if (caption == null) {
                    createCardValues(name, company, position, email, phone, web, notes, downloadUrl, downloadCaptionUrl, cardCreatedResponseListener)
                } else {
                    val myStorageCaptionReference = mFireStorage.getReference("caption_images")
                    val uploadCaptionReference = myStorageCaptionReference.child(name + Timestamp(System.currentTimeMillis()))
                    val uploadCardCaptionImage = uploadCaptionReference.putFile(caption!!)
                    uploadCardCaptionImage.addOnCompleteListener { uploadTask2 ->
                        if (uploadTask.isSuccessful) {
                            downloadCaptionUrl = uploadTask2.result.downloadUrl
                            createCardValues(name, company, position, email, phone, web, notes, downloadUrl, downloadCaptionUrl, cardCreatedResponseListener)
                        } else {
                            if (uploadTask2.exception != null) {
                                val errorMessage = uploadTask.exception!!.message
                                if (errorMessage != null) {
                                    cardCreatedResponseListener.onCardCreatedFailure(errorMessage)
                                }
                            }
                        }
                    }
                }
            } else {
                if (uploadTask.exception != null) {
                    val errorMessage = uploadTask.exception!!.message
                    if (errorMessage != null) {
                        cardCreatedResponseListener.onCardCreatedFailure(errorMessage)
                    }
                }
            }
        }
    }

    fun createCardValues(name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardCreatedResponseListener: CardsRepoInterface.CardCreatedResponseListener) {
        val cardMap: MutableMap<String, Any> = mutableMapOf()
        cardMap["name"] = name
        cardMap["company"] = company
        cardMap["user_image_url"] = profile.toString()
        if (mAuth.currentUser != null) {
            cardMap["user_id"] = mAuth.currentUser!!.uid
        }
        cardMap["timestamp"] = FieldValue.serverTimestamp()
        cardMap["position"] = position
        cardMap["email"] = email
        cardMap["phone"] = phone
        if (!web.isEmpty()) {
            cardMap["website"] = web
        }
        if (!notes.isEmpty()) {
            cardMap["notes"] = notes
        }
        if (caption != null) {
            cardMap["caption_url"] = caption.toString()
        }
        mFireStore.collection("Cards").add(cardMap).addOnCompleteListener { setTask ->
            if (setTask.isSuccessful) {
                cardCreatedResponseListener.onCardCreatedSuccesResponse()
            } else {
                if (setTask.exception != null) {
                    val errorMessage = setTask.exception!!.message
                    if (errorMessage != null) {
                        cardCreatedResponseListener.onCardCreatedFailure(errorMessage)
                    }
                }
            }
        }
    }

    override fun addCardToContacts(idCard: String, idUser: String, cardContactAddResponseListener: CardsRepoInterface.CardContactAddResponseListener) {
        val cardDocRef = mFireStore.collection("Cards").document(idCard)
        mFireStore.runTransaction { transaction ->
            val cardEntity = transaction.get(cardDocRef).toObject(CardEntity::class.java)
            val cardsMap: HashMap<String, Boolean>
            if (cardEntity?.users != null) {
                cardEntity.users!![idUser] = true
            } else {
                cardsMap = HashMap()
                cardsMap[idUser] = true
                cardEntity?.users = cardsMap
            }
            transaction.update(cardDocRef, "users", cardEntity!!.users)
            null
        }.addOnSuccessListener {
            Log.d("updateCard", "Transaction success!")
            cardContactAddResponseListener.onCardContactAddSuccesResponse()
        }.addOnFailureListener { e ->
            Log.w("updateCard", "Transaction failure.", e)
            cardContactAddResponseListener.onCardContactAddFailure(e.message!!)
        }
    }

    override fun editCard(id: String, name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardEditResponseListener: CardsRepoInterface.CardEditResponseListener) {
        when {
            profile != null -> {
                val myStorageReference = mFireStorage.getReference("card_images")
                val uploadReference = myStorageReference.child(name + Timestamp(System.currentTimeMillis()))
                val uploadEditCardImage = uploadReference.putFile(profile)
                uploadEditCardImage.addOnCompleteListener { uploadTask ->
                    if (uploadTask.isSuccessful) {
                        val downloadUrl = uploadTask.result.downloadUrl
                        if (caption != null) {
                            editCardCaption(id, name, company, position, email, phone, web, notes, downloadUrl, caption, cardEditResponseListener)
                        } else {
                            editCardValues(id, name, company, position, email, phone, web, notes, downloadUrl, caption, cardEditResponseListener)
                        }
                    } else {
                        if (uploadTask.exception != null) {
                            val errorMessage = uploadTask.exception!!.message
                            if (errorMessage != null) {
                                cardEditResponseListener.onCardEditFailure(errorMessage)
                            }
                        }
                    }
                }
            }
            caption != null -> editCardCaption(id, name, company, position, email, phone, web, notes, profile, caption, cardEditResponseListener)
            else -> editCardValues(id, name, company, position, email, phone, web, notes, profile, caption, cardEditResponseListener)
        }
    }

    fun editCardCaption(id: String, name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardEditResponseListener: CardsRepoInterface.CardEditResponseListener) {
        val myStorageReferenceCaption = mFireStorage.getReference("caption_images")
        val uploadReferenceCaption = myStorageReferenceCaption.child(name + Timestamp(System.currentTimeMillis()))
        val uploadEditCardCaptionImage = uploadReferenceCaption.putFile(caption!!)
        uploadEditCardCaptionImage.addOnCompleteListener { uploadCaptionTask ->
            if (uploadCaptionTask.isSuccessful) {
                val downloadUrlCaption = uploadCaptionTask.result.downloadUrl
                editCardValues(id, name, company, position, email, phone, web, notes, profile, downloadUrlCaption, cardEditResponseListener)
            } else {
                if (uploadCaptionTask.exception != null) {
                    val errorMessage = uploadCaptionTask.exception!!.message
                    if (errorMessage != null) {
                        cardEditResponseListener.onCardEditFailure(errorMessage)
                    }
                }
            }
        }
    }

    fun editCardValues(id: String, name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardEditResponseListener: CardsRepoInterface.CardEditResponseListener) {
        val cardMap: MutableMap<String, Any> = mutableMapOf()
        val cardDocRef = mFireStore.collection("Cards").document(id)
        mFireStore.runTransaction { transaction ->
            val cardEntity = transaction.get(cardDocRef).toObject(CardEntity::class.java)
            if (cardEntity != null) {
                cardMap["name"] = name
                cardMap["company"] = company
                cardMap["position"] = position
                cardMap["email"] = email
                cardMap["phone"] = phone
                if (!web.isEmpty()) {
                    cardMap["website"] = web
                }
                if (!notes.isEmpty()) {
                    cardMap["notes"] = notes
                }
                if (profile != null) {
                    cardMap["user_image_url"] = profile.toString()
                }
                if (caption != null) {
                    cardMap["caption_url"] = caption.toString()
                }
                transaction.update(cardDocRef, cardMap)
            }
            null
        }.addOnSuccessListener {
            Log.d("updateCard", "Transaction success!")
            cardEditResponseListener.onCardEditSuccesResponse(cardMap)
        }.addOnFailureListener { e ->
            Log.w("updateCard", "Transaction failure.", e)
            cardEditResponseListener.onCardEditFailure(e.message!!)
        }
    }
}
