package com.dev.lavin.data.services


import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.domain2.boundaries.LoginRepoInterface
import com.google.firebase.auth.FirebaseAuth
import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class LoginService(baseRepositoryComponent: BaseRepositoryComponent) : LoginRepoInterface {

    @Inject
    lateinit var mAuth: FirebaseAuth

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun login(user: String, pass: String, loginResponseListener: LoginRepoInterface.LoginResponseListener) {
        mAuth.signInWithEmailAndPassword(user, pass)
                .addOnCompleteListener { task ->
                    if (task.isSuccessful) {
                        loginResponseListener.onLoginSuccesResponse()
                    } else {
                        if (task.exception != null) {
                            val errorMessage = task.exception!!.message
                            if (errorMessage != null) {
                                loginResponseListener.onLoginFailureAuth(errorMessage)
                            }
                        }
                    }
                }
    }
}
