package com.dev.lavin.data.services


import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.data.entities.mappers.UserToUserEntityMapper
import com.dev.lavin.data.entities.user.UserEntity
import com.dev.lavin.domain2.boundaries.ProfileRepoInterface
import com.dev.lavin.domain2.model.User
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.FirebaseFirestore
import javax.inject.Inject


/**
 * Created by Jorge Lavin on 16/04/18.
 */

class ProfileService(baseRepositoryComponent: BaseRepositoryComponent) : ProfileRepoInterface {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var mFireStore: FirebaseFirestore

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun getProfileInfo(id: String, profileResponseListener: ProfileRepoInterface.ProfileResponseListener) {
        mFireStore.collection("Users").document(id).get().addOnCompleteListener { info ->
            if (info.isSuccessful) {
                if (!info.result.exists()) {
                    profileResponseListener.onProfileUserNotFound()
                } else {
                    val name = info.result.getString("name")
                    val image = info.result.getString("image")
                    val userData = UserEntity(name, image)
                    val mapper = UserToUserEntityMapper()
                    val userResult: User = mapper.reverseMap(userData)
                    profileResponseListener.onProfileSuccesResponse(userResult)
                }
            } else {
                if (info.exception != null) {
                    val errorMessage = info.exception!!.message
                    if (errorMessage != null) {
                        profileResponseListener.onProfileFailureAuth(errorMessage)
                    }
                }
            }
        }
    }
}
