package com.dev.lavin.data.services


import android.net.Uri
import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.domain2.boundaries.RegisterRepoInterface
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.storage.FirebaseStorage
import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class RegisterService(baseRepositoryComponent: BaseRepositoryComponent) : RegisterRepoInterface {

    @Inject
    lateinit var mAuth: FirebaseAuth

    @Inject
    lateinit var mFireStore: FirebaseFirestore

    @Inject
    lateinit var mFireStorage: FirebaseStorage

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun register(user: String, pass: String, name: String, url: Uri, registerResponseListener: RegisterRepoInterface.RegisterResponseListener) {
        mAuth.createUserWithEmailAndPassword(user, pass)
                .addOnCompleteListener { task ->
                    if (task.isSuccessful) {
                        //Get the currently created user object
                        val currentlyCreatedUser = task.result.user
                        //First Upload the user profile image to Firebase Storage and retrieve the uri for the image
                        val myStorageReference = mFireStorage.getReference("profile_images")
                        val uploadReference = myStorageReference.child(currentlyCreatedUser.uid)
                        val uploadProfileImage = uploadReference.putFile(url)
                        uploadProfileImage.addOnCompleteListener { uploadTask ->
                            if (uploadTask.isSuccessful) {
                                val downloadUrl = uploadTask.result.downloadUrl
                                val userMap: MutableMap<String, Any> = mutableMapOf()
                                userMap["name"] = name
                                userMap["image"] = downloadUrl.toString()
                                mFireStore.collection("Users").document(currentlyCreatedUser.uid).set(userMap).addOnCompleteListener { setTask ->
                                    if (setTask.isSuccessful) {
                                        registerResponseListener.onRegisterSuccesResponse()
                                    } else {
                                        if (task.exception != null) {
                                            val errorMessage = task.exception!!.message
                                            if (errorMessage != null) {
                                                registerResponseListener.onRegisterFailureAuth(errorMessage)
                                            }
                                        }
                                    }
                                }
                            } else {
                                if (task.exception != null) {
                                    val errorMessage = task.exception!!.message
                                    if (errorMessage != null) {
                                        registerResponseListener.onRegisterImageFailure(errorMessage)
                                    }
                                }
                            }
                        }
                    } else {
                        if (task.exception != null) {
                            val errorMessage = task.exception!!.message
                            if (errorMessage != null) {
                                registerResponseListener.onRegisterFailureAuth(errorMessage)
                            }
                        }
                    }
                }
    }

}
