package com.dev.lavin.data.services


import com.dev.lavin.data.di.BaseRepositoryComponent
import com.dev.lavin.domain2.boundaries.ResetPasswordRepoInterface
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.FirebaseAuthInvalidUserException
import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class ResetPasswordService(baseRepositoryComponent: BaseRepositoryComponent) : ResetPasswordRepoInterface {


    @Inject
    lateinit var mAuth: FirebaseAuth

    init {
        baseRepositoryComponent.inject(this)
    }

    override fun resetPassword(email: String, resetPasswordResponseListener: ResetPasswordRepoInterface.ResetPasswordResponseListener) {
        mAuth.sendPasswordResetEmail(email).addOnCompleteListener { task ->
            if (task.isSuccessful) {
                resetPasswordResponseListener.onResetPasswordSuccesResponse()
            } else {
                if (task.exception != null) {
                    if (task.exception is FirebaseAuthInvalidUserException) {
                        resetPasswordResponseListener.onResetPasswordUserNotFound()
                    } else {
                        val errorMessage = task.exception!!.message
                        if (errorMessage != null) {
                            resetPasswordResponseListener.onResetPasswordFailureAuth(errorMessage)
                        }
                    }
                }
            }
        }
    }
}
