package com.dev.lavin.domain2.boundaries

import android.net.Uri
import com.dev.lavin.domain2.model.Card


/**
 * Created by Jorge Lavin on 16/04/18.
 */

interface CardsRepoInterface {

    fun getCards(cardsResponseListener: CardsResponseListener)

    fun getContactCards(id: String,
                        cardsContactResponseListener: CardsContactResponseListener)

    fun getUserCards(id: String,
                     cardsFromUserResponseListener: CardsFromUserResponseListener)

    fun getCardsFromEvent(id: String,
                          cardsFromEventrResponseListener: CardsFromEventrResponseListener)

    fun setCardInEvent(idEvent: String,
                       idCard: String,
                       cardInEventResponseListener: CardInEventResponseListener)

    fun createCard(name: String,
                   company: String,
                   position: String,
                   email: String,
                   phone: String,
                   web: String,
                   notes: String,
                   profile: Uri?,
                   caption: Uri?,
                   cardCreatedResponseListener: CardCreatedResponseListener)

    fun addCardToContacts(idCard: String,
                          idUser: String,
                          cardContactAddResponseListener: CardsRepoInterface.CardContactAddResponseListener)

    fun editCard(id: String,
                 name: String,
                 company: String,
                 position: String,
                 email: String,
                 phone: String,
                 web: String,
                 notes: String,
                 profile: Uri?,
                 caption: Uri?,
                 cardEditResponseListener: CardEditResponseListener)


    interface CardsResponseListener {

        fun onCardsSuccesResponse(cardList: MutableMap<String, MutableList<Card>>)

        fun onCardsFailure(message: String)
    }

    interface CardsContactResponseListener {

        fun onCardsContactSuccesResponse(cardList: MutableMap<String, MutableList<Card>>)

        fun onCardsContactFailure(message: String)
    }

    interface CardsFromUserResponseListener {

        fun onCardsFromUserSuccesResponse(cardList: MutableMap<String, MutableList<Card>>)

        fun onCardsFromUserFailure(message: String)

        fun onCardsFromUserEmpty()
    }

    interface CardsFromEventrResponseListener {

        fun onCardsFromEventSuccesResponse(cardList: MutableMap<String, MutableList<Card>>)

        fun onCardsFromEventFailure(message: String)

        fun onCardsFromEventEmpty()
    }

    interface CardInEventResponseListener {

        fun onCardInEventSuccesResponse()

        fun onCardInEventFailure(message: String)
    }

    interface CardCreatedResponseListener {

        fun onCardCreatedSuccesResponse()

        fun onCardCreatedFailure(message: String)
    }

    interface CardContactAddResponseListener {

        fun onCardContactAddSuccesResponse()

        fun onCardContactAddFailure(message: String)
    }

    interface CardEditResponseListener {

        fun onCardEditSuccesResponse(cardMap: MutableMap<String, Any>)

        fun onCardEditFailure(message: String)
    }
}
