package com.dev.lavin.domain2.interactors.cards

import android.net.Uri
import com.dev.lavin.domain2.boundaries.CardsRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent
import com.dev.lavin.domain2.model.Card
import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class CardsInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : CardsInteractor {

    @Inject
    lateinit var cardsRepoInterface: CardsRepoInterface

    init {
        baseInteractorComponent.inject(this)
    }

    override fun getCards(cardsInteractorResponse: CardsInteractor.CardsInteractorResponse) {
        val cardsRunnable = CardsRunnable(cardsInteractorResponse)
        cardsRunnable.run()
    }

    override fun getContactCards(id: String, cardsContactInteractorResponse: CardsInteractor.CardsContactInteractorResponse) {
        val cardsContactRunnable = CardsContactRunnable(id, cardsContactInteractorResponse)
        cardsContactRunnable.run()
    }

    override fun getUserCards(id: String, cardsFromUserInteractorResponse: CardsInteractor.CardsFromUserInteractorResponse) {
        val cardsFromUserRunnable = CardsFromUserRunnable(id, cardsFromUserInteractorResponse)
        cardsFromUserRunnable.run()
    }

    override fun getCardsFromEvent(key: String, cardFromEventInteractorResponse: CardsInteractor.CardFromEventInteractorResponse) {
        val cardsFromEventRunnable = CardsFromEventRunnable(key, cardFromEventInteractorResponse)
        cardsFromEventRunnable.run()
    }

    override fun setCardInEvent(idEvent: String, idCard: String, cardsSetInEventInteractorResponse: CardsInteractor.CardsSetInEventInteractorResponse) {
        val cardsInEventRunnable = CardsInEventRunnable(idEvent, idCard, cardsSetInEventInteractorResponse)
        cardsInEventRunnable.run()
    }

    override fun createCard(name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, createCardsInteractorResponse: CardsInteractor.CreateCardsInteractorResponse) {
        val createCardsEventRunnable = CreateCardsEventRunnable(name, company, position, email, phone, web, notes, profile, caption, createCardsInteractorResponse)
        createCardsEventRunnable.run()
    }

    override fun addCardToContacts(idCard: String, idUser: String, cardsContactsAddInteractorResponse: CardsInteractor.CardsContactsAddInteractorResponse) {
        val addContactCardEventRunnable = AddContactCardEventRunnable(idCard, idUser, cardsContactsAddInteractorResponse)
        addContactCardEventRunnable.run()
    }

    override fun editCard(id: String, name: String, company: String, position: String, email: String, phone: String, web: String, notes: String, profile: Uri?, caption: Uri?, cardEditInteractorResponse: CardsInteractor.CardEditInteractorResponse) {
        val editCardsEventRunnable = EditCardsEventRunnable(id, name, company, position, email, phone, web, notes, profile, caption, cardEditInteractorResponse)
        editCardsEventRunnable.run()
    }

    private inner class CardsRunnable(private val cardsInteractorResponse: CardsInteractor.CardsInteractorResponse) : Runnable, CardsRepoInterface.CardsResponseListener {

        override fun run() {
            cardsRepoInterface.getCards(this)
        }

        override fun onCardsSuccesResponse(cardList: MutableMap<String, MutableList<Card>>) {
            cardsInteractorResponse.onCardsInteractorResponseSuccess(cardList)
        }

        override fun onCardsFailure(message: String) {
            cardsInteractorResponse.onCardsInteractorResponseFailure(message)
        }
    }

    private inner class CardsContactRunnable(private val id: String, private val cardsContactInteractorResponse: CardsInteractor.CardsContactInteractorResponse) : Runnable, CardsRepoInterface.CardsContactResponseListener {

        override fun run() {
            cardsRepoInterface.getContactCards(id, this)
        }

        override fun onCardsContactSuccesResponse(cardList: MutableMap<String, MutableList<Card>>) {
            cardsContactInteractorResponse.onCardsContactInteractorResponseSuccess(cardList)
        }

        override fun onCardsContactFailure(message: String) {
            cardsContactInteractorResponse.onCardsContactInteractorResponseFailure(message)
        }
    }

    private inner class CardsFromUserRunnable(private val id: String, private val cardsFromUserInteractorResponse: CardsInteractor.CardsFromUserInteractorResponse) : Runnable, CardsRepoInterface.CardsFromUserResponseListener {

        override fun run() {
            cardsRepoInterface.getUserCards(id, this)
        }

        override fun onCardsFromUserSuccesResponse(cardList: MutableMap<String, MutableList<Card>>) {
            cardsFromUserInteractorResponse.onCardsFromUserInteractorResponseSuccess(cardList)
        }

        override fun onCardsFromUserFailure(message: String) {
            cardsFromUserInteractorResponse.onCardsFromUserInteractorResponseFailure(message)
        }

        override fun onCardsFromUserEmpty() {
            cardsFromUserInteractorResponse.onCardsFromUserEmpty()
        }
    }

    private inner class CardsFromEventRunnable(private val id: String, private val cardFromEventInteractorResponse: CardsInteractor.CardFromEventInteractorResponse) : Runnable, CardsRepoInterface.CardsFromEventrResponseListener {

        override fun run() {
            cardsRepoInterface.getCardsFromEvent(id, this)
        }

        override fun onCardsFromEventFailure(message: String) {
            cardFromEventInteractorResponse.onCardFromEventInteractorResponseFailure(message)
        }

        override fun onCardsFromEventSuccesResponse(cardList: MutableMap<String, MutableList<Card>>) {
            cardFromEventInteractorResponse.onCardFromEventInteractorResponseSuccess(cardList)
        }

        override fun onCardsFromEventEmpty() {
            cardFromEventInteractorResponse.onCardsFromEventEmpty()
        }
    }

    private inner class CardsInEventRunnable(private val idEvent: String, private val idCard: String, private val cardsSetInEventInteractorResponse: CardsInteractor.CardsSetInEventInteractorResponse) : Runnable, CardsRepoInterface.CardInEventResponseListener {

        override fun run() {
            cardsRepoInterface.setCardInEvent(idEvent, idCard, this)
        }

        override fun onCardInEventFailure(message: String) {
            cardsSetInEventInteractorResponse.onCardsSetInEventInteractorResponseFailure(message)
        }

        override fun onCardInEventSuccesResponse() {
            cardsSetInEventInteractorResponse.onCardsSetInEventInteractorResponseSuccess()
        }
    }

    private inner class CreateCardsEventRunnable(private val name: String, private val company: String, private val position: String, private val email: String, private val phone: String, private val web: String, private val notes: String, private val profile: Uri?, private val caption: Uri?, private val createCardsInteractorResponse: CardsInteractor.CreateCardsInteractorResponse) : Runnable, CardsRepoInterface.CardCreatedResponseListener {

        override fun run() {
            cardsRepoInterface.createCard(name, company, position, email, phone, web, notes, profile, caption, this)
        }

        override fun onCardCreatedSuccesResponse() {
            createCardsInteractorResponse.onCreateCardsInteractorResponseSuccess()
        }

        override fun onCardCreatedFailure(message: String) {
            createCardsInteractorResponse.onCreateCardsInteractorResponseFailure(message)
        }
    }

    private inner class AddContactCardEventRunnable(private val idCard: String, private val idUser: String, private val cardsContactsAddInteractorResponse: CardsInteractor.CardsContactsAddInteractorResponse) : Runnable, CardsRepoInterface.CardContactAddResponseListener {

        override fun run() {
            cardsRepoInterface.addCardToContacts(idCard, idUser, this)
        }

        override fun onCardContactAddSuccesResponse() {
            cardsContactsAddInteractorResponse.onCardsContactsAddInteractorResponseSuccess()
        }

        override fun onCardContactAddFailure(message: String) {
            cardsContactsAddInteractorResponse.onCardsContactsAddInteractorResponseFailure(message)
        }

    }

    private inner class EditCardsEventRunnable(private val id: String, private val name: String, private val company: String, private val position: String, private val email: String, private val phone: String, private val web: String, private val notes: String, private val profile: Uri?, private val caption: Uri?, private val cardEditInteractorResponse: CardsInteractor.CardEditInteractorResponse) : Runnable, CardsRepoInterface.CardEditResponseListener {

        override fun run() {
            cardsRepoInterface.editCard(id, name, company, position, email, phone, web, notes, profile, caption, this)
        }

        override fun onCardEditSuccesResponse(cardMap: MutableMap<String, Any>) {
            cardEditInteractorResponse.onCardEditInteractorResponseSuccess(cardMap)
        }

        override fun onCardEditFailure(message: String) {
            cardEditInteractorResponse.onCardEditInteractorResponseFailure(message)
        }
    }
}
