package com.dev.lavin.domain2.interactors.events

import android.net.Uri
import com.dev.lavin.domain2.boundaries.EventsRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent
import com.dev.lavin.domain2.model.Event
import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class EventsInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : EventsInteractor {

    @Inject
    lateinit var eventsRepoInterface: EventsRepoInterface

    init {
        baseInteractorComponent.inject(this)
    }

    override fun getEvents(eventsInteractorResponse: EventsInteractor.EventsInteractorResponse) {
        val eventsRunnable = EventsRunnable(eventsInteractorResponse)
        eventsRunnable.run()
    }

    override fun getEventDetail(id: String, eventDetailInteractorResponse: EventsInteractor.EventDetailInteractorResponse) {
        val eventsDetailRunnable = EventsDetailRunnable(id, eventDetailInteractorResponse)
        eventsDetailRunnable.run()
    }

    override fun createEvent(name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri, eventsCreateInteractorResponse: EventsInteractor.EventsCreateInteractorResponse) {
        val createEventRunnable = EventsCreateRunnable(name, date, time, location, category, description, url, eventsCreateInteractorResponse)
        createEventRunnable.run()
    }

    override fun editEvent(id: String, name: String, date: Long, time: String, location: String, category: String, description: String, url: Uri?, eventsEditInteractorResponse: EventsInteractor.EventsEditInteractorResponse) {
        val createEditRunnable = EventsEditRunnable(id, name, date, time, location, category, description, url, eventsEditInteractorResponse)
        createEditRunnable.run()
    }

    private inner class EventsRunnable(private val eventsInteractorResponse: EventsInteractor.EventsInteractorResponse) : Runnable, EventsRepoInterface.EventsResponseListener {

        override fun run() {
            eventsRepoInterface.getEvents(this)
        }

        override fun onEventsFailure(message: String) {
            eventsInteractorResponse.onEventsInteractorResponseFailure(message)
        }

        override fun onEventsSuccesResponse(event: MutableMap<String, MutableList<Event>>) {
            eventsInteractorResponse.onEventsInteractorResponseSuccess(event)
        }

        override fun onEventsEmptyResponse() {
            eventsInteractorResponse.onEventsEmptyInteractorResponse()
        }
    }

    private inner class EventsDetailRunnable(private val id: String, private val eventDetailInteractorResponse: EventsInteractor.EventDetailInteractorResponse) : Runnable, EventsRepoInterface.EventDetailResponseListener {

        override fun run() {
            eventsRepoInterface.getEventDetail(id, this)
        }

        override fun onEventDetailSuccesResponse(event: Event) {
            eventDetailInteractorResponse.onEventDetailInteractorResponseSuccess(event)
        }

        override fun onEventDetaiFailure(message: String) {
            eventDetailInteractorResponse.onEventDetailInteractorResponseFailure(message)
        }

        override fun onEventNotExistResponse() {
            eventDetailInteractorResponse.onEventNotExistInteractorResponse()
        }
    }

    private inner class EventsCreateRunnable(private val name: String, private val date: Long, private val time: String, private val location: String, private val category: String, private val description: String, private val url: Uri, private val eventsCreateInteractorResponse: EventsInteractor.EventsCreateInteractorResponse) : Runnable, EventsRepoInterface.EventsCreateResponseListener {

        override fun run() {
            eventsRepoInterface.createEvent(name, date, time, location, category, description, url, this)
        }

        override fun onEventsCreateSuccesResponse() {
            eventsCreateInteractorResponse.onEventsCreateInteractorResponseSuccess()
        }

        override fun onEventsCreateFailure(message: String) {
            eventsCreateInteractorResponse.onEventsCreateInteractorResponseFailure(message)
        }

    }

    private inner class EventsEditRunnable(private val id: String, private val name: String, private val date: Long, private val time: String, private val location: String, private val category: String, private val description: String, private val url: Uri?, private val eventsEditInteractorResponse: EventsInteractor.EventsEditInteractorResponse) : Runnable, EventsRepoInterface.EventsEditResponseListener {

        override fun run() {
            eventsRepoInterface.editEvent(id, name, date, time, location, category, description, url, this)
        }

        override fun onEventsEditSuccesResponse(eventMap: MutableMap<String, Any>) {
            eventsEditInteractorResponse.onEventsEditInteractorResponseSuccess(eventMap)
        }

        override fun onEventsEditFailure(message: String) {
            eventsEditInteractorResponse.onEventsEditInteractorResponseFailure(message)
        }

    }

}
