package com.dev.lavin.domain2.interactors.login

import com.dev.lavin.domain2.boundaries.LoginRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent

import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class LoginInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : LoginInteractor {

    @Inject
    lateinit var loginRepoInterface: LoginRepoInterface


    init {
        baseInteractorComponent.inject(this)
    }

    override fun login(user: String, pass: String, loginInteractorResponse: LoginInteractor.LoginInteractorResponse) {
        val loginRunnable = LoginRunnable(user, pass, loginInteractorResponse)
        loginRunnable.run()
    }

    private inner class LoginRunnable (private val user: String, private val pass: String, private val loginInteractorResponse: LoginInteractor.LoginInteractorResponse) : Runnable, LoginRepoInterface.LoginResponseListener {

        override fun run() {
            loginRepoInterface.login(user, pass, this)
        }

        override fun onLoginSuccesResponse() {
            loginInteractorResponse.onLoginInteractorResponseSuccess()
        }

        override fun onLoginFailureAuth(message: String) {
            loginInteractorResponse.onLoginInteractorResponseFailureAuth(message)
        }
    }
}
