package com.dev.lavin.domain2.interactors.profile

import com.dev.lavin.domain2.boundaries.ProfileRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent
import com.dev.lavin.domain2.model.User

import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class ProfileInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : ProfileInteractor {

    @Inject
    lateinit var profileRepoInterface: ProfileRepoInterface

    init {
        baseInteractorComponent.inject(this)
    }

    override fun getProfileInfo(id: String, profileInteractorResponse: ProfileInteractor.ProfileInteractorResponse) {
        val profileRunnable = ProfileRunnable(id, profileInteractorResponse)
        profileRunnable.run()
    }

    private inner class ProfileRunnable (private val id: String, private val profileInteractorResponse: ProfileInteractor.ProfileInteractorResponse) : Runnable, ProfileRepoInterface.ProfileResponseListener {

        override fun run() {
            profileRepoInterface.getProfileInfo(id, this)
        }


        override fun onProfileUserNotFound() {
            profileInteractorResponse.onProfileUserNotFound()
        }

        override fun onProfileFailureAuth(message: String) {
            profileInteractorResponse.onProfileFailureAuth(message)
        }

        override fun onProfileSuccesResponse(user: User) {
            profileInteractorResponse.onProfileSuccesResponse(user)
        }
    }

}
