package com.dev.lavin.domain2.interactors.register

import android.net.Uri

import com.dev.lavin.domain2.boundaries.RegisterRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent

import javax.inject.Inject

/**
 * Created by Jorge Lavin on 16/04/18.
 */

class RegisterInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : RegisterInteractor {

    @Inject
    lateinit var registerRepoInterface: RegisterRepoInterface

    init {
        baseInteractorComponent.inject(this)
    }

    override fun register(user: String, pass: String, name: String, url: Uri, registerInteractorResponse: RegisterInteractor.RegisterInteractorResponse) {
        val registerRunnable = RegisterRunnable(user, pass, name, url, registerInteractorResponse)
        registerRunnable.run()
    }

    private inner class RegisterRunnable (private val user: String, private val pass: String, private val name: String, private val url: Uri, private val registerInteractorResponse: RegisterInteractor.RegisterInteractorResponse) : Runnable, RegisterRepoInterface.RegisterResponseListener {

        override fun run() {
            registerRepoInterface.register(user, pass, name, url, this)
        }

        override fun onRegisterSuccesResponse() {
            registerInteractorResponse.onRegisterInteractorSuccesResponse()
        }

        override fun onRegisterFailureAuth(message: String) {
            registerInteractorResponse.onRegisterInteractorFailureAuth(message)
        }

        override fun onRegisterImageFailure(message: String) {
            registerInteractorResponse.onRegisterInteractorFailureAuth(message)
        }
    }
}
