package com.dev.lavin.domain2.interactors.resetpassword

import com.dev.lavin.domain2.boundaries.ResetPasswordRepoInterface
import com.dev.lavin.domain2.di.BaseInteractorComponent
import javax.inject.Inject

/**
 * Created by lavin
 * on 28/04/2018.
 */
class ResetPasswordInteractorImpl(baseInteractorComponent: BaseInteractorComponent) : ResetPasswordInteractor {

    @Inject
    lateinit var resetPasswordRepoInterface: ResetPasswordRepoInterface

    init {
        baseInteractorComponent.inject(this)
    }

    override fun resetPassword(email: String, resetPasswordInteractorResponse: ResetPasswordInteractor.ResetPasswordInteractorResponse) {
        val resetPasswordRunnable = ResetPasswordRunnable(email, resetPasswordInteractorResponse)
        resetPasswordRunnable.run()
    }

    private inner class ResetPasswordRunnable(private val email: String, private val resetPasswordInteractorResponse: ResetPasswordInteractor.ResetPasswordInteractorResponse) : Runnable, ResetPasswordRepoInterface.ResetPasswordResponseListener {

        override fun run() {
            resetPasswordRepoInterface.resetPassword(email, this)
        }

        override fun onResetPasswordSuccesResponse() {
            resetPasswordInteractorResponse.onResetPasswordSuccesResponse()
        }

        override fun onResetPasswordFailureAuth(message: String) {
            resetPasswordInteractorResponse.onResetPasswordFailure(message)
        }

        override fun onResetPasswordUserNotFound() {
            resetPasswordInteractorResponse.onResetPasswordUserNotFound()
        }
    }

}